#===============================================================
#
#   fail.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package fail;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
FAIL_filespec
FAIL_log
FAIL_read
);

use strict;
use integer;


use glo::spit;
use glo::slurp;
use gbsenv;




sub FAIL_filespec();
sub FAIL_log($$$$$);
sub FAIL_read();

sub get_filespec();




my $FAIL_FILE_SPEC;




sub FAIL_filespec()
{
get_filespec()
if (!defined $FAIL_FILE_SPEC);


return $FAIL_FILE_SPEC;
}




sub FAIL_log($$$$$)
{
my ($command_name,
$rc,
$subsys,
$component,
$file,
) = @_;

$subsys = '-'
if (!$subsys);
$component = '-'
if (!$component);
$file = '-'
if (!$file);

get_filespec()
if (!defined $FAIL_FILE_SPEC);

SPIT_file( ">$FAIL_FILE_SPEC",
"**FAILED: $command_name ($rc): $subsys $component $file\n");
}




sub FAIL_read()
{
my @lines;

get_filespec()
if (!defined $FAIL_FILE_SPEC);

if (-f $FAIL_FILE_SPEC)
{
@lines = SLURP_file( $FAIL_FILE_SPEC);
}

return @lines;
}




sub get_filespec()
{
if (GBSENV_mode_is_background())
{
$FAIL_FILE_SPEC = "$ENV{GBS_LOGFILE}.err";
} else
{
my $gbs_root_path = GBSENV_gbs_root_path();
my $gbs_exec_id = GBSENV_get_exec_id();
$FAIL_FILE_SPEC = join( '_', "$gbs_root_path/tmp/fail", "$gbs_exec_id.err");
}

}

return 1;



