#===============================================================
#
#   gbsaudit.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

print "*GBSAUDIT @ARGV\n" if ($ENV{GBSDEBUG_FILE});
use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::time;
use glo::genopt;
use gbsenv;
use gbsres;
use gbsoptenv;
use validate;
use swa;
use run;
use gbsexec;
use gbsbg;
use bld;
use cfo;
use gfl;
use audit;
use fspec;




sub do_audits();
sub do_audit($);
sub do_summaries();
sub do_summary($);
sub show_summary_file_names();
sub get_sum_filespec($$);
sub get_datetime();




my $GBS_ROOT_PATH;
my $GBS_SUBSYS;
my $GBS_COMPONENT;
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_AUDITS = ENV_getenv( 'GBS_AUDITS');
my $CWD;

my $GBS_SUBSYS_PATH;
my $GBS_AUDIT_PATH;
my $GBS_COMP_PATH;
my $GBS_TMP_PATH;

my $NOW = time();

my $HTML_LOG_DIR;
my @ALL_AUDIT_NAMES;
my @SUM_FILES;




my $MUST_SHOW_STDOUT = 0;






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<*>',    'comp_file_opts',              'sam',  '', "src-files and AUDIT-options" ],
[ 'i',      'ignore_errors',               'bso',   0, "Continue auditing after error(s)" ],
[ 'stdout', 'show_stdout',	               'bso',   1, "Show errorlines (0 when not interactive)" ],
[ 'vs',     'view_sum',	               'bso',   1, "View summary at completion" ],
[ 'audits', 'audit_names', "saos$GBS_AUDITS", $GBS_AUDIT, "(Wild-)Audits to run" ],
[ 'aud',    'audit', 	               'bso',   1, "Run audit" ],
[ 'sum',    'summary',	               'bso',   1, "Create summary" ],
[ 'call',   'call_count',	               'iso',   0, "Internal use. 0=normal, 1=split_first, <0=split_last" ],
[ 'jobs',	'jobs',		          'isor1..9',   2, 'Max. nr. parallel jobs' ],
);
my @genenvs = qw( LOG_PATH MODE FLAGS_* APP_* );
GENOPT_set_optdefs( 'gbsaudit', \@genopts,
'Perform an AUDIT on one or more files',
undef);
GENOPT_set_envdefs( \&GBSOPTENV_set, \&GBSOPTENV_print, 'GBS_', \@genenvs);
GENOPT_parse();
my @COMP_FILE_OPTS = GENOPT_get( 'comp_file_opts');
my $IGNORE_ERRORS = GENOPT_get( 'ignore_errors');
my $SHOW_STDOUT = GENOPT_get( 'show_stdout');
my $MUST_VIEW_SUM = GENOPT_get( 'view_sum');
my @AUDIT_NAMES = GENOPT_get( 'audit_names');
my $MUST_DO_AUDIT = GENOPT_get( 'audit');
my $MUST_DO_SUM = GENOPT_get( 'summary');
my $CALL_COUNT = GENOPT_get( 'call_count');
my $JOBS = GENOPT_get( 'jobs');




ENV_setenv( GBS_IGNORE_ERRORS => $IGNORE_ERRORS);
$MUST_SHOW_STDOUT = (GBSENV_mode_is_interactive() && $SHOW_STDOUT) ? 1 : 0;
ENV_setenv( GBS_AUDIT_OPTIONS => [ $MUST_SHOW_STDOUT ]);




















$CWD = ENV_cwd();
if ($CALL_COUNT >= 0)
{



($GBS_ROOT_PATH, $GBS_SUBSYS) = VALIDATE_subsys_audit();
$GBS_COMPONENT = ENV_getenv('GBS_COMPONENT');
VALIDATE_target();

$GBS_SUBSYS_PATH = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS";
$GBS_AUDIT_PATH = "$GBS_SUBSYS_PATH/audit";
$GBS_COMP_PATH = "$GBS_SUBSYS_PATH/comp";
} else
{



ENV_say( 1, "Summary");
$GBS_ROOT_PATH = VALIDATE_root();
}





$GBS_TMP_PATH = "$GBS_ROOT_PATH/tmp";

@ALL_AUDIT_NAMES = VALIDATE_audits( @AUDIT_NAMES);




my $OK_TO_CONTINUE = 1;
GBSBG_batch_init( $JOBS);
if ($CALL_COUNT == 0)
{



$HTML_LOG_DIR = ENV_getenv_perl_path( 'GBS_LOG_PATH');
my $this_rc = do_audits();
$RC = $this_rc
if ($this_rc > $RC);
show_summary_file_names();
} else
{



$HTML_LOG_DIR = "$GBS_ROOT_PATH/silo/audits";
mkdir $HTML_LOG_DIR
if (!-e $HTML_LOG_DIR);

if ($CALL_COUNT >= 1)
{
$MUST_DO_SUM = 0;
my $this_rc = do_audits();
$RC = $this_rc
if ($this_rc > $RC);
} else  # call_count < 0
{



if ($MUST_DO_SUM)
{
my $this_rc = do_summaries();
$RC = $this_rc
if ($this_rc > $RC);
}
show_summary_file_names();
}
}


GBSBG_batch_finish();








{
my $audit = ENV_getenv( 'GBS_AUDIT');

SWA_set( $audit, 1);

if ($audit ne $GBS_AUDIT)
{
my @lines;
push @lines, GBSENV_changed_setenv_commands( 0, 'GBS_AUDIT', 'GBS_AUDIT_PLUGIN');

GBSENV_write_result_script( \@lines, 0);
}
}

ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 1);
}




sub do_audits()
{
my $rc = 0;




foreach my $audit (@ALL_AUDIT_NAMES)
{
my $this_rc = do_audit( $audit);
$rc = $this_rc
if ($this_rc > $rc);
}

return $rc;
}




sub do_audit($)
{
my ($audit,
) = @_;
my $rc = 0;

my $nr_cfo = CFO_parse_audit( $audit, \@COMP_FILE_OPTS);
if ($nr_cfo > 0)
{
if ($MUST_DO_AUDIT)
{
ENV_say( 1, "Executing 'Audit'...");




$rc = GBSEXEC_audit( $audit, $nr_cfo, $IGNORE_ERRORS);
$RC = $rc
if ($rc > $RC);
} else
{
ENV_say( 1, "$audit: Skipped (--audit-)");
BLD_init_audit( $audit);	# Sets: GBS_AUDIT and AUDIT_envs
}




my $this_gfl = GFL_new( $audit, [ get_datetime() ]);
GFL_put_subsys( $this_gfl, $GBS_SUBSYS, undef);

for (my $i = 0; $i < $nr_cfo; $i++)
{



my (undef, $component, $files_ref, $opt_ref) = CFO_get_item( $i);


GFL_put_component( $this_gfl, $component, undef);
GFL_put_files( $this_gfl, $files_ref);
}
GFL_write( $this_gfl);




if ($MUST_DO_SUM)
{
if ($rc == 0 || $IGNORE_ERRORS)
{
do_summary( $audit);
} else
{
ENV_say( 1, "Summary not executed (rc=$rc)");
}
}
} else
{
ENV_warn( "Nothing to do for $audit");
}
return $rc;
}




sub do_summaries()
{
my $rc = 0;

foreach my $audit (@ALL_AUDIT_NAMES)
{
my $this_rc = do_summary( $audit);
$rc = $this_rc if ($this_rc > $rc);
}

return $rc;
}




sub do_summary($)
{
my ($audit) = @_;
my $rc = 0;

my ($app, $app_data_ref) = GFL_read_head( $audit);
if (defined $app)
{
my $audit_plugin = SWA_set( $audit, 0);
my $sum_filespec = get_sum_filespec( $audit, $app_data_ref->[0]);  # $datetime_packed);




{
my @lines;
if (-e FSPEC_scadef( $audit_plugin))
{
push @lines, GBSBG_banner( "Audit SCA Summary: $audit");
push @lines, GBSBG_scasum( "${audit}_summary", $sum_filespec);
} else
{
my @pos_args = ($sum_filespec);
my @command_data_refs = AUDIT_get_summary_command_data_refs_exp( $audit, \@pos_args);
if (@command_data_refs)
{
push @lines, GBSBG_banner( "Audit Summary: $audit");
push @lines, GBSBG_exec( "${audit}_summary", undef, undef, @command_data_refs);	# $must_print_commands, $out_files
} else
{
ENV_say( 1, "$audit: No Summary Command specified in audit.gbs");
}
}
if (@lines)
{
$rc = GBSBG_batch_run( 'gbsaudit_post', '-', \@lines);
ENV_say( 1, "$audit: Summary generated ($rc)...")
if (-e $sum_filespec);
}
}




if (-e $sum_filespec)
{
if ($MUST_VIEW_SUM)
{
ENV_say( 1, "$audit: View Summary file");
ENV_fatal( "$audit: Summary file not found '$sum_filespec'")
if (!-f $sum_filespec);
push @SUM_FILES, $sum_filespec;
ENV_say( 1, "$audit: Starting Browser...");
RUN_browser( $sum_filespec);
}
} else
{
ENV_say( 1, "$audit: No Summary generated ($rc)...");
}
unlink( GFL_filespec( $audit));
} else
{
ENV_say( 1, "$audit: No Summary generated...");
}

return $rc;
}




sub show_summary_file_names()
{
if (@SUM_FILES)
{
ENV_say( 1, "Summary files:");
map { ENV_say( 2, "  $_") } @SUM_FILES;
}
}




sub get_sum_filespec($$)
{
my ($audit,
$audit_datetime_packed,
) = @_;


my $audit_name = ($CALL_COUNT == 0) ? 'gbsaudit' : 'gbssysaudit';
my $audit_prefix = join( '_', $audit_name, $GBS_TARGET, $audit, $audit_datetime_packed);

return "$HTML_LOG_DIR/${audit_prefix}_sum.html";
}




sub get_datetime()
{
my $audit_datetime_packed;

if (GBSENV_mode_is_background())
{



my $GBS_LOGFILE = ENV_getenv( 'GBS_LOGFILE');
ENV_fatal( "GBS_LOGFILE not set in background!")
if ($GBS_LOGFILE eq '');
my (undef, undef, undef, $numtime_packed) = GBSRES_split_logfile( $GBS_LOGFILE);
$audit_datetime_packed = $numtime_packed;
} else
{
$audit_datetime_packed = TIME_time2num_packed( $NOW);
}

return $audit_datetime_packed;
}


