#===============================================================
#
#   gbsdb.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package gbsdb;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
GBSDB_last_root_get
GBSDB_last_root_write
GBSDB_roots_get
GBSDB_roots_write
GBSDB_maps_get
GBSDB_maps_write
);

use strict;
use integer;

use glo::env;
use glo::spit;
use fspec;




sub GBSDB_last_root_get();
sub GBSDB_roots_get();
sub GBSDB_maps_get();
sub GBSDB_last_root_write($);
sub GBSDB_roots_write($);
sub GBSDB_maps_write($);

sub read_file($);
sub write_file();




my $GBSDB_FILESPEC = FSPEC_gbsdb();

my $CURRENT_FILE = '';
my %OS_LAST_ROOT_PATH;	    # last_root-$os=$last_root_path

my %OS_ROOT_PATH_REFS;	    # root-$os=$root_path

my @MAP_REFS;		    # maps=$win_drive,$unix_path





sub GBSDB_last_root_get()
{
my $last_root_path;

read_file( $GBSDB_FILESPEC);
$last_root_path = $OS_LAST_ROOT_PATH{$^O};

return (defined $last_root_path) ? $last_root_path : '';
}




sub GBSDB_roots_get()
{
read_file( $GBSDB_FILESPEC);

return @{$OS_ROOT_PATH_REFS{$^O}};
}





sub GBSDB_maps_get()
{
read_file( $GBSDB_FILESPEC);

return @MAP_REFS;
}




sub GBSDB_last_root_write($)
{
my ($last_root_path) = @_;

if ($last_root_path ne '')
{
my $current_last_root_path = GBSDB_last_root_get();
if ($last_root_path ne $current_last_root_path)
{
$OS_LAST_ROOT_PATH{$^O} = $last_root_path;
write_file();
}
}
}




sub GBSDB_roots_write($)
{
my ($root_paths_ref) = @_;

read_file( $GBSDB_FILESPEC);
$OS_ROOT_PATH_REFS{$^O} = [ sort( @{$root_paths_ref}) ];
write_file();
}





sub GBSDB_maps_write($)
{
my ($map_refs_ref) = @_;

read_file( $GBSDB_FILESPEC);
@MAP_REFS = @{$map_refs_ref};	# keep original order!
write_file();
}




sub read_file($)
{
my ($file_spec) = @_;

if ($file_spec ne $CURRENT_FILE)
{



%OS_ROOT_PATH_REFS = ();
$OS_ROOT_PATH_REFS{$^O} = [];
@MAP_REFS = ();


if (open( IN, $file_spec))
{
my @lines = <IN>;
close IN;
foreach my $line (@lines)
{
$line =~ s/\s+$//;			# remove trailing white space
next if ($line eq "");			# skip empty lines
next if (substr( $line, 0, 1) eq "#");	# skip comments
my ($name, $value) = split( /=/, $line, 2);
$value = '' if (!defined $value);
if ($name eq 'maps')
{



my ($win_drive, $unix_path) = split( ',', $value, 2);
push @MAP_REFS, [ $win_drive, $unix_path ];
} elsif (substr( $name, 0, 6) eq 'roots-')
{



my $os = substr( $name, 6);
push @{$OS_ROOT_PATH_REFS{$os}}, $value;
} elsif (substr( $name, 0, 10) eq 'last_root-')
{



my $os = substr( $name, 10);
$OS_LAST_ROOT_PATH{$os} = $value;
} elsif ($name eq 'id')
{



} else
{
ENV_warn( "Invalid key '$name' in file $file_spec");
}
}
} else
{

}

$CURRENT_FILE = $file_spec;
}
}





sub write_file()
{
my @lines;
push @lines, "id=gbsdb\n";

foreach my $os (sort( keys (%OS_LAST_ROOT_PATH)))
{
push @lines, "last_root-$os=$OS_LAST_ROOT_PATH{$os}\n";
}

foreach my $os (sort( keys (%OS_ROOT_PATH_REFS)))
{
push @lines, map { "roots-$os=$_\n" } @{$OS_ROOT_PATH_REFS{$os}};
}

foreach my $ref (@MAP_REFS)
{
my ($drive, $dir) = @{$ref};
push @lines, "maps=$drive,$dir\n";
}



SPIT_file( $CURRENT_FILE, \@lines);
}

return 1;
