#==============================================================
#
#   gbsfilecom.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package gbsfilecom;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
GBSFILECOM_validate_path

GBSFILECOM_expand_command_data_refs
);

use strict;
use integer;

use glo::env;
use gbsfile;




sub GBSFILECOM_validate_path($);

sub GBSFILECOM_expand_command_data_refs($$$);









my $IS_WIN32 = ENV_is_win32();








sub GBSFILECOM_validate_path($)
{
my ($path) = @_;
my $full_path;

if (ENV_is_abs_path( $path))
{
my $abs_path_not_allowed = 1;
if (!$IS_WIN32)	# IS_LINUX
{
$abs_path_not_allowed = 0
if (substr( $path, 0, 5) eq '/usr/' ||
$path eq '/lib');
}
GBSFILE_error( "Absolute path not allowed, use EnvVar from switch.gbs")
if ($abs_path_not_allowed);
}

$full_path = ENV_perl_path_nospace( ENV_expand_envs( $path));
GBSFILE_error( "No such directory:", "'$full_path'")
if (!-d $full_path && substr( $full_path, 0, 5) ne '/usr/');
GBSFILE_error( "Must be derrived Abs Path:", "'$full_path'")
if (!ENV_is_abs_path_perl( $full_path));


return $full_path;
}














sub GBSFILECOM_expand_command_data_refs($$$)
{
my ($command_data_refs_ref,

$pos_args_ref,	    # Target:  [ $file_name, $in_file_spec, $out_file_spec, $out_file_dir, $out_file_type ]



$gen_args_ref,	    # Flags from flags_*.gbs files + @command_line_flags
) = @_;
my @command_data_refs;

my @pos_args = (defined $pos_args_ref) ? map { ENV_os_path( $_) } @{$pos_args_ref} : ();
my $gen_args = (defined $gen_args_ref) ? ENV_join_quoted_space( @{$gen_args_ref}) : '';


foreach my $command_data_ref (@{$command_data_refs_ref})
{
my ($command_type, $pos_arg_tokens, $command_line, $ok_values, $exec_condition) = @{$command_data_ref};


if ($pos_arg_tokens ne '0')
{
my $pos_par_token = substr( $pos_arg_tokens, 1);	    # % or \$




$command_line =~ s/$pos_par_token\*/$gen_args/g;




for (my $i = 1; $i <= @pos_args; $i++)
{
$command_line =~ s/$pos_par_token$i/$pos_args[$i-1]/g;
$exec_condition =~ s/$pos_par_token$i/$pos_args[$i-1]/g;
}
} else
{
if ($command_type ne 'B')	    # direct builtin (e.g. echo)
{
$command_line = "$command_line @pos_args"	    # join quoted string???
if (@pos_args);
$command_line = "$command_line $gen_args"	    # join quoted string???
if ($gen_args ne '');
}
}
push @command_data_refs, [ $command_type, $pos_arg_tokens, $command_line, $ok_values, $exec_condition ];

}

return @command_data_refs;
}

return 1;


