#===============================================================
#
#   gbsmaint.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

print "*GBSMAINT @ARGV*\n" if ($ENV{GBSDEBUG_FILE});
use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::ask;
use glo::genopt;
use gbsenv;
use gbsglo;
use gbscleanup;
use fix;
use owners;
use maint;
use intgrtr;
use admin;












my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');

my @MAIN_MENU_ITEMS = (
[ "Audits Handling",
\&MAINT_audits_handling ],
[ "Cleanup ",
\&GBSCLEANUP_main ],
[ "Map Network Drives (unix<->win32)",
\&MAINT_map_network_drives ],
[ "Not used",
sub { ENV_say( 1, "Not used!") } ],
[ "Not used",
sub { ENV_say( 1, "Not used!") } ],
[ "Integrator Tools",
\&INTGTR_main ],
[ "Administrator Tools",
\&ADMIN_main ],
[ "Check/Fix GBS Directory Structure",
\&FIX_main ],
[ "'Owners-file' Maintenance",
\&OWNERS_maintenance ]
);






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>', 'menu_entry_1', "isor1..9", 0, "Primary menu entry for single immediate execution" ],
[ '<2>', 'menu_entry_2', "iso>0",    0, "Secondary menu entry for single immediate execution" ],
[ '<3>', 'menu_entry_3', "iso>0",    0, "3rd menu entry for single immediate execution" ],
[ '<4>', 'menu_entry_4', "iso>0",    0, "4th menu entry for single immediate execution" ],
[ '<5>', 'menu_entry_5', "iso>0",    0, "5th menu entry for single immediate execution" ],
);
GENOPT_set_optdefs( 'gbsmaint', \@genopts,
[ 'Allow user to execute General Maintenance functions',
' 1. Audits Handling',
' 2. Cleanup',
' 3. Map Network Drives (unix<->win32)',
' 6. Integrator Tools',
' 7. Administrator Tools',
' 8. Check/Fix GBS Directory Structure',
' 9. Owners-file Maintenance',
],
undef);
GENOPT_parse();
my $MENU_ENTRY_1 = GENOPT_get( 'menu_entry_1');
my $MENU_ENTRY_2 = GENOPT_get( 'menu_entry_2');
my $MENU_ENTRY_3 = GENOPT_get( 'menu_entry_3');
my $MENU_ENTRY_4 = GENOPT_get( 'menu_entry_4');
my $MENU_ENTRY_5 = GENOPT_get( 'menu_entry_5');
my @MENU_ENTRIES = grep( $_ != 0, ($MENU_ENTRY_1, $MENU_ENTRY_2, $MENU_ENTRY_3, $MENU_ENTRY_4, $MENU_ENTRY_5)) ;

map { $_--} @MENU_ENTRIES;

ENV_pushd( undef);

if ($GBS_COMPONENT ne '')
{
ENV_chdir( "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp/$GBS_COMPONENT/src");
} elsif ($GBS_SUBSYS ne '')
{
ENV_chdir( "$GBS_ROOT_PATH/dev/$GBS_SUBSYS");
} elsif ($GBS_ROOT_PATH ne '')
{
ENV_warn( "No current SubSystem");
ENV_chdir( "$GBS_ROOT_PATH");
} else
{
ENV_warn( "No current GBS_ROOT_PATH. You are on your own!");
}

if ($GBS_ROOT_PATH ne '' && $GBS_SUBSYS ne '' && !GBSGLO_subsystem_is_full_gbs( $GBS_SUBSYS))
{
ENV_info( "Subsystem ($GBS_SUBSYS) is not full-GBS",
"Functions may not be available");
}




{
ENV_say( 1, "General GBS Maintenance");

ASK_menu( 'Select function to perform',
\@MAIN_MENU_ITEMS, [ @MENU_ENTRIES ]);
}




ENV_popd();




{
if (ENV_get_changed_envs())
{
my @lines;
push @lines, GBSENV_changed_setenv_commands( 0);

GBSENV_write_result_script( \@lines, 0);
}
}

ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 1);
}


