#===============================================================
#
#   gbsres.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package gbsres;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(
GBSRES_title
GBSRES_title_html
GBSRES_company_html
GBSRES_numtime_packed
GBSRES_logfile
GBSRES_split_logfile
GBSRES_skiptypes
);

use strict;
use integer;

use glo::env;
use glo::time;




sub GBSRES_title();
sub GBSRES_title_html();
sub GBSRES_company_html();
sub GBSRES_numtime_packed();
sub GBSRES_logfile($$$);
sub GBSRES_split_logfile($);
sub GBSRES_skiptypes();




my $TIME;		    # Do not initialize!
my $NUMTIME_PACKED;	    # Do not initialize!

my $TITLE = "G E N E R I C    B U I L D    S U P P O R T";
my $COMPANY = "Randy Marques Consultancy";





sub GBSRES_title()
{
return "$TITLE  -  $COMPANY";
}




sub GBSRES_title_html()
{
my $title = $TITLE;
$title =~ s/   /&nbsp;&nbsp;&nbsp;/g;

return $title;
}




sub GBSRES_company_html()
{
my $company = $COMPANY;
$company =~ s/ /&nbsp;/g;

return $company;
}




sub GBSRES_numtime_packed()
{
if (!defined $NUMTIME_PACKED)
{
$TIME = time
if (!defined $TIME);
$NUMTIME_PACKED = TIME_time2num_packed( $TIME);
}
return $NUMTIME_PACKED;
}




sub GBSRES_logfile($$$)
{
my ($jobname,
$target,
$action,
) = @_;
my $logfile_spec;

my $gbs_log_dir = ENV_getenv_perl_path( 'GBS_LOG_PATH');

my $target_name = ($target eq '-') ? 'Target' : $target;
my $action_name = ($action eq '-') ? 'Audit'  : $action;
my $log_name = join( '_', ($jobname, $target_name, $action_name, $NUMTIME_PACKED));
$logfile_spec = "$gbs_log_dir/$log_name.log";

return $logfile_spec;
}




sub GBSRES_split_logfile($)
{
my ($logfile_spec) = @_;
my ($jobname, $target, $audit, $numtime);

my $logfile = ENV_split_spec_n( $logfile_spec);
( $jobname, $target, $audit, $numtime ) = $logfile =~ /^([^_]+)_(.*)_(.*)_(\d\d\d\d\d\d\d\d-\d\d\d\d\d\d)$/;
ENV_fatal( "Unknown logfile name format '$logfile_spec'")
if (!defined $jobname);

return ($jobname, $target, $audit, $numtime);
}




sub GBSRES_skiptypes()
{
return qw! (^|/)~ ~$ \.sav\d*$ \.bak$ \.bck$ \.old$ \.tmp$ \.\$+$ \.proto$ (^|/)Thumbs.db$ !;
}

return 1;
