#===============================================================
#
#   gbssilo.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package gbssilo;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
GBSSILO_create_index
GBSSILO_view_index
);
use strict;
use integer;

use glo::env;
use glo::html;
use glo::spit;
use glo::slurp;
use glo::time;
use glo::file;
use gbsenv;
use gbshtml;
use run;




sub GBSSILO_create_index();
sub GBSSILO_view_index();

sub do_topdir($);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SILO_PATH = "$GBS_ROOT_PATH/silo";
my $INDEX_FILE_SPEC = "$GBS_SILO_PATH/index.html";

my $GBS_SYSTEM_NAME = ENV_getenv( 'GBS_SYSTEM_NAME');
my $GBS_LOG_PATH = ENV_getenv( 'GBS_LOG_PATH');



my @SECTION_REFS;





sub GBSSILO_create_index()
{
ENV_say( 1, 'Creating Index...');




{
my @section = ('GBS Log Overviews');

my $gen_sum_file_spec = "$GBS_SILO_PATH/.gbs/gbssum/build.html";
my $audit_sum_file_spec = "$GBS_SILO_PATH/.gbs/gbssum/audit.html";
push @section, [ $gen_sum_file_spec, 'Generate/Make (.log)']
if (-e $gen_sum_file_spec);
push @section, [ $audit_sum_file_spec, 'Audits (.log & .html)' ]
if (-e $audit_sum_file_spec);
push @SECTION_REFS, [ @section ];
}




foreach my $dir (SLURP_dir_dirs( $GBS_SILO_PATH, 0))
{
do_topdir( $dir);
}




my @lines;




my $title = "Silo - $GBS_SYSTEM_NAME";

push @lines, GBSHTML_doc_start( $INDEX_FILE_SPEC, $title, 0);
push @lines, GBSHTML_doc_top( [ $title,  HTML_link( $GBS_LOG_PATH, HTML_bold( 'GBS Log Directory'))], ());
my $section = 1;




my $nr_sections = @SECTION_REFS;
if ($nr_sections > 0)
{
foreach my $ref (@SECTION_REFS)
{
my ($section_name, @item_refs) = @{$ref};
my $next_section = ($section < $nr_sections) ? $section++ : -$section;
push @lines, GBSHTML_section_head( $section_name, $next_section);
if (@item_refs)
{
my @li_lines;
foreach my $item_ref (@item_refs)
{
my ($file_spec, $title) = @{$item_ref};
if (-d $file_spec)
{
push @li_lines, HTML_li( HTML_link( $file_spec, $title));
} else
{
my $datetime = TIME_time2num( FILE_mtime( $file_spec));
push @li_lines, HTML_li( HTML_link( $file_spec, $title), HTML_txt( '   '), HTML_italic( $datetime));
}
}
push @lines, HTML_ul( @li_lines);
} else
{
push @lines, "No Files";
}
}
} else
{
push @lines, "No Data";
}




push @lines, GBSHTML_doc_bottom( $title, $section);
push @lines, GBSHTML_doc_end();

SPIT_file_nl( $INDEX_FILE_SPEC, \@lines);
ENV_say( 1, "Created Silo index");
}




sub do_topdir($)
{
my ($dir) = @_;


ENV_say( 2, "TopDir $dir...");

my $this_path = "$GBS_SILO_PATH/$dir";
my @section = (ucfirst( $dir));

if (-f "$this_path/index.html")
{
push @section, [ "$this_path/index.html", "$dir/index.html" ];
} else
{
my @subdirs = SLURP_dir_dirs( $this_path, 0);
if (@subdirs)
{
foreach my $subdir (@subdirs)
{

if (-f "$this_path/$subdir/index.html")
{
push @section, [ "$this_path/$subdir/index.html", "$dir/$subdir/index.html" ];
} else
{
push @section, [ "$this_path/$subdir", "$dir/$subdir" ];
}
}
} else
{
push @section, [ "$this_path", "$dir" ];
}
}
push @SECTION_REFS, [ @section ];
}




sub GBSSILO_view_index()
{
RUN_browser( $INDEX_FILE_SPEC);
}

return 1;


