#===============================================================
#
#   gbssub.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

print "*GBSSUB*\n" if ($ENV{GBSDEBUG_FILE});
use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::args;
use gbsenv;
use gbscmd;
use fail;
use target;
use bldopt;
use exec;
use run;




sub get_command_args();








my ($JOBTYPE,	    #  audit|gen|make|cleanp_bld|cleanup_aud
$IGNORE_ERRORS, # bool
);

my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_SSTYPE = ENV_getenv( 'GBS_SSTYPE');

my $GBS_AUDIT;	    # DO not initilaize!

my $FOR_GEN = 0;
my $FOR_MAKE = 0;
my $FOR_AUDIT = 0;
my $FOR_CLEANUP_BLD = 0;
my $FOR_CLEANUP_AUD = 0;
my $FOR_BUILD = 0;	    # gen | make
my $FOR_CLEANUP = 0;	    # cleaup_aud | cleaup_build
















$| = 1;
my $RC = 0;




GBSENV_init();

get_command_args();

if ($JOBTYPE eq 'gen')
{
$FOR_GEN = 1;
$FOR_BUILD = 1;
} elsif ($JOBTYPE eq 'make')
{
$FOR_MAKE = 1;
$FOR_BUILD = 1;
} elsif ($JOBTYPE eq 'audit')
{
$FOR_AUDIT = 1;
$GBS_AUDIT  = ENV_getenv( 'GBS_AUDIT');
} elsif ($JOBTYPE eq 'cleanup_bld')
{
$FOR_CLEANUP_BLD = 1;
$FOR_CLEANUP = 1;
} elsif ($JOBTYPE eq 'cleanup_aud')
{
$FOR_CLEANUP_AUD = 1;
$FOR_CLEANUP = 1;
$GBS_AUDIT  = ENV_getenv( 'GBS_AUDIT');
} else
{
ENV_fatal( "Invalid JOBTYPE '$JOBTYPE'");
}

ENV_setenv( GBS_IGNORE_ERRORS => $IGNORE_ERRORS);
ENV_setenv( GBS_MAKE => GBSCMD_make());




{
my $options_banner = BLDOPT_init( $GBS_AUDIT);



if ($FOR_BUILD || $FOR_AUDIT)	    # exclude FOR_CLEANUPs
{



my @env_def_refs;
foreach my $src_type (TARGET_get_src_types( $GBS_TARGET))
{
my $out_file_base = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/bldopt$src_type";
push @env_def_refs, BLDOPT_get_envs_settings( '', $out_file_base, $src_type, undef);
}
map { ENV_setenv( $_->[0], $_->[1]) } @env_def_refs;
if (0)
{
print "GBSSUB: BEGIN ENV_DEFS\n";
map { print "  $_->[0]\t=> $_->[1]\n" } @env_def_refs;
print "GBSSUB: END ENV_DEFS\n";
}
}




my $gbs_ignore_errors = ENV_getenv( 'GBS_IGNORE_ERRORS');
if ($GBS_SSTYPE eq 'make')
{
my @make_flags;
push @make_flags, '-i'
if ($gbs_ignore_errors);
$ENV{GBS_MAKE_FLAGS} = "@make_flags";	    # we do not want to use ENV_setenv because of the "" if empty
} elsif ($GBS_SSTYPE eq 'MSVS')
{
my $gbs_mode = ENV_getenv( 'GBS_MODE');
my $msbuild_config = ($gbs_mode eq 'FINAL') ? 'Release' : ($gbs_mode eq 'DEBUG') ? 'Debug' : '';
$ENV{GBS_MSBUILD_CONFIG} = $msbuild_config; # we do not want to use ENV_setenv because of the "" if empty
} elsif ($GBS_SSTYPE eq 'Other')
{

} else
{
ENV_error_exit( "Invalid SubSys Type '$GBS_SSTYPE");
}




ENV_say( 1, "** $options_banner");




$RC = RUN_gbssub_script( $JOBTYPE);
ENV_say( 0, '-');
FAIL_log( "gbssub$JOBTYPE", $RC, $GBS_SUBSYS, '', '')
if ($RC != 0);

if ($FOR_BUILD)
{



if (-d "$GBS_ROOT_PATH/res/$GBS_SUBSYS" || -d "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/export")
{
if ($IGNORE_ERRORS || $RC == 0)
{
ENV_say( 1, "Implicit gbsexport of $GBS_SUBSYS");
my @gbsexport_opts;
push @gbsexport_opts, '--i' if ($IGNORE_ERRORS);
my $this_rc = EXEC_run( 'gbsexport.pl', \@gbsexport_opts);
$RC = $this_rc if ($this_rc > $RC);
FAIL_log( 'gbsexport', $this_rc, $GBS_SUBSYS, '', '')
if ($this_rc != 0);
} else
{
ENV_say( 1, "SKIPPED: gbsexport of $GBS_SUBSYS");
}
}
}
}




ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 1);
}




sub get_command_args()
{


($JOBTYPE,	     #  audit|gen|make|cleanp_bld|cleanup_aud
$IGNORE_ERRORS, # bool
) = ARGS_get( 2, 'perl gbssub.pl audit|gen|make|cleanp_bld|cleanup_aud <ignore_errors>', undef);
}



