#==============================================================
#
#   gbsswr_set.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
print "*GBSSWR_SET @ARGV\n" if ($ENV{GBSDEBUG_FILE});
use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::shell;
use glo::genopt;
use gbsenv;
use gbscmd;
use gbsglo;
use system;
use swt;
use swa;
use swc;
use sws;
use swr;
use swglo;
use gbsdb;








my $CWD = ENV_cwd();
my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();

my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SYSTEM_NAME = ENV_getenv( 'GBS_SYSTEM_NAME');
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ 'new',	'new_root',	   'bso',	0, "Must create new Target" ],
[ 'exec',	'execute_command', 'sso',      '', "(GBS-)Command(s) to execute. 'none' = OK" ],
[ 'nocd',	'no_cd',	   'bso',	0, "Do not 'cd' to the selected directories "],
);
my @genconflicts = (
);
GENOPT_set_optdefs( [ 'gbsswr_set', '' ], \@genopts,
'Internal Use',
undef);
GENOPT_set_conflicts( \@genconflicts);
GENOPT_parse();
my $NEW = GENOPT_get( 'new_root');
my $EXEC = GENOPT_get( 'execute_command');
my $NO_CD = GENOPT_get( 'no_cd');




{



{
my $must_write = ($GBS_ROOT_PATH ne '');
SWR_set( $GBS_ROOT_PATH, $must_write);
}
{
my $must_write = ($GBS_SUBSYS ne '');
SWS_set( $GBS_SUBSYS, $must_write);
}
{
my $must_write = ($GBS_COMPONENT ne '');
SWC_set( $GBS_SUBSYS, $GBS_COMPONENT, $must_write);
}
{
SWT_set( $GBS_TARGET, 1);
SWA_set( $GBS_AUDIT, 1);
}

if ($GBS_ROOT_PATH ne '')
{
if (GBSENV_mode_is_interactive())
{
GBSDB_last_root_write( $GBS_ROOT_PATH);

ENV_warn( "No current SubSystem")
if ($GBS_SUBSYS eq '');
ENV_warn( "No current Component")
if ($GBS_COMPONENT eq '' && GBSGLO_subsystem_is_full_gbs( $GBS_SUBSYS));
ENV_warn( "No current Target")
if ($GBS_TARGET eq '');
}
} else
{
ENV_warn( "No current Root");
}




{



my @lines;




if ($GBS_ROOT_PATH ne '')
{
push @lines, SHELL_echo( '')
if ($GBS_ROOT_PATH ne '');




push @lines, GBSENV_changed_setenv_commands( 0);	# $must_log = 0




push @lines, SWGLO_set_title( $GBS_ROOT_PATH, $GBS_SUBSYS, $GBS_COMPONENT);
}




my $new_cwd = ($NO_CD) ? $CWD : SWGLO_new_cwd( $GBS_ROOT_PATH, $GBS_SUBSYS, $GBS_COMPONENT);
push @lines, SHELL_conditional_eq( GBS_RC => 0, SHELL_cd( $new_cwd));




push @lines, SHELL_conditional_ne( GBS_RC => 0, SHELL_setenv( GBS_ROOT_PATH => '""'),
SHELL_setenv( GBS_SUBSYS => '""'),
SHELL_setenv( GBS_COMPONENT => '""'),
SHELL_setenv( GBS_TARGET => '""'),
SHELL_setenv( GBS_AUDIT => '""'),
SWR_cleanup(),
SWGLO_set_title(),
SHELL_cd( $GBS_ROOT_PATH));




push @lines, SHELL_conditional_eq( GBS_RC => 0, GBSCMD_command( 'gbsshow', '--brief'));




push @lines, SHELL_conditional_ne( GBS_RC => 0, SHELL_echo_failed());

if ($NEW)
{



my @this_platform_targets = SYSTEM_get_os_ref_names( $GBS_ROOT_PATH, '*', 'targets');

push @lines, SHELL_conditional_eq( GBS_RC => 0, GBSCMD_command( 'gbsswt', '--new'))
if (@this_platform_targets == 0);
} elsif ($EXEC ne '' && $EXEC ne 'none')
{



my ($command, @rest) = split( ' ', $EXEC);
$command = 'gbsshow'
if ($command eq 'gbs');
my $script_command = ENV_commandfile_spec( "$GBS_SCRIPTS_PATH/$command");   # TBS --brief ????
my $command_ref;
if (-f $script_command)
{
$command_ref = ENV_command_ref( $script_command, \@rest);
} elsif (-f "$GBS_SCRIPTS_PATH/$command.pl")
{
$command_ref = ENV_command_ref( "$command.pl", \@rest);
} else
{
ENV_error_exit( "Invalid/Empty '--exec' command: $EXEC");
}
my $exec_command = ENV_command_line( $command_ref);


push @lines, SHELL_conditional_eq( GBS_RC => 0, $exec_command);
push @lines, SHELL_setrc();
}

push @lines, SHELL_exit_rc( 1);
GBSENV_write_result_script( \@lines, 0);	# $print_lines = 0
}
}

ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 0);
}


