#===============================================================
#
#   gbssysaudit.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

print "*GBSSYSAUDIT*\n" if ($ENV{GBSDEBUG_FILE});
use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::genopt;
use gbsenv;
use gbsoptenv;
use validate;
use gbssys;
use swa;




sub submit();
sub execute();




my $GBS_ROOT_PATH;






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<*>',	 'steps',	  'sao',      "", "Steps and/or Aliases. <empty> == all select steps, 'ALL' == force all steps, '?' == show steps" ],
[ 'i' ,	 'ignore_errors', 'bso',       0, "Continue generation after error(s)" ],
[ 'vs',	 'view_sum',	  'bso',       1, "View summary at completion" ],
[ 'audits',  'audits',	  'sao',   '-,*', "(wild-)Audits to run. '' or '*' == All, '.' == Current, '-' == NonSpecific" ],
[ 'targets', 'targets',	  'sao',   '-,*', "(wild-)Targets to Audit. '' or '*' == All, '.' == Current, '-' == NonSpecific" ],
[ 'files',	 'files',	  'sao', '*:*.*', "Files to audit" ],
[ 'aud',	 'audit',	  'bso',       1, "Run audit" ],
[ 'sum',	 'summary',	  'bso',       1, "Create summary" ],
[ 'fg',	 'foreground',	  'bso',       0, "Runs in the foreground if set" ],
[ 'at',	 'delay',	  'tso',   'now', "Starting time (10:20), delta (+10:10) or now" ],
[ 'sm',	 'mail',	  'bso',       0, "Send mail on completion" ],
[ 'n',	 'notify',	  'bso',       1, "Notify user on completion (bg/nowait only)" ],
[ 'wait',    'wait',          'bso',       0, "Wait for completion of bg jobs" ],
[ 'jobs',	 'jobs',     'isor1..9',       2, 'Max nr parallel jobs within a submitted job' ],
[ 'c',	 'comment',	  'sso',      "", 'Comments - will be shown in Summary - Specify %20 for space' ],
);
my @genconflicts = (
[ [ fg   => 1 ], '=' => 'at', '=' => 'sm', '=' => 'n' ],
[ [ wait => 1 ], '=' => 'n' ],
[ [ sum  => 0 ], '=', [ vs => 1 ] ],
);
my @genenvs = qw( LOG_PATH FLAGS_* APP_* );
GENOPT_set_optdefs( 'gbssysaudit', \@genopts,
'Perform an AUDIT on part of or whole System',
undef);
GENOPT_set_conflicts( \@genconflicts);
GENOPT_set_envdefs( \&GBSOPTENV_set, \&GBSOPTENV_print, 'GBS_', \@genenvs);
GENOPT_parse();
my @STEPS = GENOPT_get( 'steps');
my $IGNORE_ERRORS = GENOPT_get( 'ignore_errors');
my $MUST_VIEW_SUM = GENOPT_get( 'view_sum');
my @AUDITS = GENOPT_get( 'audits');
my @TARGETS = GENOPT_get( 'targets');
my @FILES = GENOPT_get( 'files');
my $DO_AUDIT = GENOPT_get( 'audit');
my $DO_SUM = GENOPT_get( 'summary');
my $FOREGROUND = GENOPT_get( 'foreground');
my $DELAY = GENOPT_get( 'delay');
my $MAIL = GENOPT_get( 'mail');
my $NOTIFY = GENOPT_get( 'notify');
my $WAIT = GENOPT_get( 'wait');
my $JOBS = GENOPT_get( 'jobs');

my $COMMENT = GENOPT_get( 'comment');




$NOTIFY = 0
if ($WAIT);




$GBS_ROOT_PATH = VALIDATE_root();

GBSSYS_init(
'audit',
$IGNORE_ERRORS,
\@TARGETS,
\@AUDITS,
\@FILES,
$FOREGROUND,
[ $DELAY, $MAIL, $NOTIFY, $COMMENT ],
$WAIT,
);

if (GBSENV_mode_is_interactive())
{



$RC = submit();

} else
{



$RC = execute();
}




ENV_exit( $RC);




sub END
{
my $rc = $?;




GBSSYS_print_errors()
if ($rc != 0);

ENV_print_end_msg( 1);
}




sub submit()
{
my @excludes = qw( steps targets audits files delay wait comment);
my @opts = GENOPT_get_changed( \@excludes );

return GBSSYS_submit( \@STEPS, \@opts);
}




sub execute()
{
my $rc = 0;

my $audit = $AUDITS[0];	# executes 1 audit for 1 target
my @gbsaudit_opts =
(
GENOPT_makeopt( ignore_errors => $IGNORE_ERRORS),
GENOPT_makeopt( view_sum => $MUST_VIEW_SUM),
GENOPT_makeopt( audits => $audit),			# only 1 audit
GENOPT_makeopt( audit => $DO_AUDIT),
GENOPT_makeopt( summary => $DO_SUM),
GENOPT_makeopt( jobs => $JOBS),
);

SWA_set( $audit, 0);					# set GBS_AUDIT, etc
$rc = GBSSYS_execute( \@STEPS, \@gbsaudit_opts);

return $rc;
}


