#===============================================================
#
#   args.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package glo::args;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
ARGS_get
ARGS_error_exit
ARGS_fatal
);

use strict;
use integer;

use glo::env;
use glo::check;




sub ARGS_get($$$);
sub ARGS_error_exit(@);
sub ARGS_fatal(@);

sub print_usage();




my $USAGE;




sub ARGS_get($$$)
{
my ($limits,    # undef = any; value = exact, ref = [min,max]
$usage,	    #
$help,	    #
) = @_;
my @argv;	    # or @ARGV

$USAGE = $usage;




if (@ARGV == 1)
{
if ($ARGV[0] eq '--h')
{
print_usage();
exit(0);
} elsif ($ARGV[0] eq '--help')
{
print_usage();
if (defined $help)
{
ENV_say( 0, $help);
} else
{
ENV_say( 0, '*No additional help*');
}
exit(0);
}
}




if (defined $limits)
{
my $nr_args = @ARGV;
my $error_text = CHECK_num_value( 'nr ARGuments', $nr_args, $limits);
if (defined $error_text)
{
ENV_say( 0, "ARGV=@ARGV");
ARGS_error_exit( $error_text);
}
}




if (ENV_is_win32())
{
foreach my $arg (@ARGV)
{
my @files = ENV_glob( $arg);

push @argv, (@files) ? @files : $arg;
}
return @argv;
} else
{
return @ARGV;
}
}




sub ARGS_error_exit(@)
{
my @lines = @_;

print_usage();
ENV_say( 0, @lines);
exit( 2);
}




sub ARGS_fatal(@)
{
my @lines = @_;

print_usage();
ENV_say( 0, @lines);
ENV_stackdump();
exit( 9);
}




sub print_usage()
{
ENV_say( 0, "Usage: $USAGE");
}

return 1;

