#===============================================================
#
#   html.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#
#   Based on HTML 4.01
#   <!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
#
#   http://www.w3.org/TR/html401/
#
#===============================================================

package glo::html;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
HTML_map
HTML_doc_s
HTML_doc_e
HTML_body_s
HTML_body_e
HTML_anchor
HTML_llink
HTML_link
HTML_img
HTML_hr
HTML_br
HTML_ul
HTML_ol
HTML_li
HTML_table
HTML_table_s
HTML_table_e
HTML_tr
HTML_trth
HTML_trtd
HTML_th
HTML_td
HTML_p
HTML_div
HTML_span
HTML_h
HTML_bold
HTML_bold_txt
HTML_italic
HTML_italic_txt
HTML_txt
HTML_pre_s
HTML_pre_txt
HTML_pre_e
);

use strict;
use integer;

use glo::env;




sub HTML_map(@);
sub HTML_doc_s($$;@);
sub HTML_doc_e();
sub HTML_body_s(@);
sub HTML_body_e();
sub HTML_anchor($$);
sub HTML_llink($$);
sub HTML_link($$;$);
sub HTML_img($;@);
sub HTML_hr(@);
sub HTML_table($$$;@);
sub HTML_table_s(@);
sub HTML_table_e();
sub HTML_tr(@);
sub HTML_trth(@);
sub HTML_trtd(@);
sub HTML_th(@);
sub HTML_td(@);
sub HTML_br(@);
sub HTML_ul(@);
sub HTML_ol(@);
sub HTML_li(@);
sub HTML_p(@);
sub HTML_div(@);
sub HTML_span(@);
sub HTML_h($@);
sub HTML_bold(@);
sub HTML_bold_txt(@);
sub HTML_italic(@);
sub HTML_italic_txt(@);
sub HTML_txt(@);
sub HTML_pre_s(@);
sub HTML_pre_txt(@);
sub HTML_pre_e();

sub table_rows($$$$);
sub generic_align_4($@);
sub assert($$);
sub map_url($$);
sub fix_name($);
sub indent($);




my @align_3	= qw( left center right );
my @align_4	= qw( left center right justify);
my @align_5	= qw( left center right justify char );
my @align_img	= qw( bottom middle top left right );
my @clear	= qw( left all right none );
my @frame	= qw( void above below hsides vsides lhs rhs box border );
my @rules	= qw( none groups rows cols all );
my @scope	= qw( row col rowgroup colgroup );
my @target	= qw( _blank _self _parent _top );
my @valign	= qw( top middle bottom baseline );










my %TYPES = (
abbr	=> 't', 	    # TD, TH
align_3	=> \@align_3,	    # HR, TABLE
align_4	=> \@align_4,	    # DIV, SPAN, Hx, P
align_5	=> \@align_5,	    # COL, COLGROUP, TBODY, TD, TFOOT, TH, THEAD, TR
align_img	=> \@align_img,     # IMG
alt 	=> 't', 	    # APPLET, IFRAME, IMG, INPUT, OBJECT
axis	=> 't', 	    # TD, TH
bgcolor	=> 'c', 	    # TABLE, TD, TH, TD, BODY
border_table=> 'n', 	    # TABLE
border_img	=> 'l', 	    # IMG, OBJECT
cellpadding => 'l', 	    # TABLE
cellspacing => 'l', 	    # TABLE
char	=> 's', 	    # COL, COLGROUP, TBODY, TD, TFOOT, TH, THEAD, TR
charoff	=> 'l', 	    # COL, COLGROUP, TBODY, TD, TFOOT, TH, THEAD, TR
class	=> 't', 	    # NOT { BASE, BASEFONT, HEAD, HTML, META, PARAM, SCRIPT, STYLE, TITLE }
clear	=> \@clear,	    # BR
colspan	=> 'n', 	    # TD TH
css 	=> 'u', 	    # DOC
cssprint	=> 'u',		    # DOC
favicon 	=> 'u', 	    # DOC
frame	=> \@frame,	    # TABLE
headers	=> 'i', 	    # TD TH
height_l	=> 'l', 	    # IFRAME, IMG, OBJECT, APPLET
height_n	=> 'n', 	    # TD TH
href	=> 'u', 	    # A, AREA, LINK, BASE
hspace	=> 'n', 	    # APPLET, IMG, OBJECT
id		=> 'i', 	    # NOT { BASE, HEAD, HTML, META, SCRIPT, STYLE, TITLE }
name_a	=> 't', 	    # A
noshade	=> '',		    # HR
nowrap	=> '',		    # TD TH
rowspan	=> 'n', 	    # TD TH
rules	=> \@rules,	    # TABLE
scope	=> \@scope,	    # TD TH
script	=> 'u',		    # DOC
size_n	=> 'n', 	    # HR
src 	=> 'u', 	    # SCRIPT, INPUT, FRAME, IFRAME, IMG
style	=> 't', 	    # NOT { BASE, BASEFONT, HEAD, HTML, META, PARAM, SCRIPT, STYLE, TITLE }
summary	=> 't', 	    # TABLE
target	=> \@target,	    # A, AREA, BASE, FORM, LINK
text	=> 'c', 	    # BODY
title	=> 't', 	    # NOT {  BASE, BASEFONT, HEAD, HTML, META, PARAM, SCRIPT, STYLE }
valign	=> \@valign,	    # COL, COLGROUP, TBODY, TD, TFOOT, TH, THEAD, TR
vspace	=> 'n', 	    # APPLET, IMG, OBJECT
width_n	=> 'n', 	    # TD, TH, PRE
width_l	=> 'l', 	    # HR, IFRAME, IMG, OBJ, TABLE, APPLET,
);


my %DOC = (
css     => '',
cssprint=> '',
style   => '',
script  => '',
favicon => '',
);

my %BODY = (
id	    => '',
class   => '',
style   => '',
bgcolor => '',
text    => '',	 # deprecated
);

my %A = (
id	    => '',
class   => '',
style   => '',
target  => '',


);

my %IMG = (
id		=> '',
class	=> '',
style	=> '',

alt 	=> '',
title	=> '',
align	=> 'align_img',
width	=> 'width_l',
height	=> 'height_n',
border	=> 'border_img',
hspace	=> '',
vspace	=> '',
);

my %HR = (
id		=> '',
class	=> '',
style	=> '',
title	=> '',
align	=> 'align_3',
noshade	=> '',
size	=> 'size_n',
width	=> 'width_l',
);

my %TABLE = (
id		=> '',
class	=> '',
style	=> '',
title	=> '',
summary	=> '',
width	=> 'width_l',
bgcolor	=> '',
frame	=> '',
rules	=> '',
border	=> 'border_table',
cellspacing => '',
cellpadding => '',
align	=> 'align_3',	    #deprecated
);

my %TR = (
id		=> '',
class	=> '',
style	=> '',
title	=> '',
align	=> 'align_5',
char	=> '',
charoff	=> '',
valign	=> '',
);

my %THTD = (
id		=> '',
class	=> '',
style	=> '',
title	=> '',
headers	=> '',
scope	=> '',
abbr	=> '',
axis	=> '',
rowspan	=> '',
colspan	=> '',
nowrap	=> '',		# deprecated
width	=> 'width_l',	# deprecated
heigth	=> 'heigth_n',	# deprecated
bgcolor	=> '',
align	=> 'align_5',
char	=> '',
charoff	=> '',
valign	=> '',
);

my %BR = (
id		=> '',
class	=> '',
style	=> '',
title	=> '',
clear	=> '',		# deprecated
);

my %GENERIC_ALIGN_4 = (
id		=> '',		# generic set
class	=> '',		# generic set
style	=> '',		# generic set
title	=> '',		# generic set
align	=> 'align_4',	# deprecated
);

my %GENERIC = (
id		=> '',		# generic set
class	=> '',		# generic set
style	=> '',		# generic set
title	=> '',		# generic set
);

my %HTML_CHARS = (

'&'	=> '&amp;',	# ampersand
'>'	=> '&gt;',	# greater than
'<'	=> '&lt;',	# less than
'"'	=> '&quot;',	# double quote


chr(0)  => '&lt;NUL&gt;',
chr(1)  => '&lt;SOH&gt;',
chr(2)  => '&lt;STX&gt;',
chr(3)  => '&lt;ETX&gt;',
chr(4)  => '&lt;EOT&gt;',
chr(5)  => '&lt;ENQ&gt;',
chr(6)  => '&lt;ACK&gt;',
chr(7)  => '&lt;BEL&gt;',	# \a
chr(8)  => '&lt;BS&gt;',	# \b


chr(11) => '&lt;VT&gt;',
chr(12) => '&lt;FF&gt;',	# \f
chr(13) => '&lt;CR&gt;',	# \r
chr(14) => '&lt;SO&gt;',
chr(15) => '&lt;SI&gt;',
chr(16) => '&lt;DLE&gt;',
chr(17) => '&lt;DC1&gt;',
chr(18) => '&lt;DC2&gt;',
chr(19) => '&lt;DC3&gt;',
chr(20) => '&lt;DC4&gt;',
chr(21) => '&lt;NAK&gt;',
chr(22) => '&lt;SYN&gt;',
chr(23) => '&lt;ETB&gt;',
chr(24) => '&lt;CAN&gt;',
chr(25) => '&lt;EM&gt;',
chr(26) => '&lt;SUB&gt;',
chr(27) => '&lt;ESC&gt;',	# \e
chr(28) => '&lt;FS&gt;',
chr(29) => '&lt;GS&gt;',
chr(30) => '&lt;RS&gt;',
chr(31) => '&lt;US&gt;',

chr(127) => '&lt;DEL&gt;',


''	=> '&AElig;',	# capital AE diphthong (ligature)
''	=> '&Aacute;',	# capital A, acute accent
''	=> '&Acirc;',	# capital A, circumflex accent
''	=> '&Agrave;',	# capital A, grave accent
''	=> '&Aring;',	# capital A, ring
''	=> '&Atilde;',	# capital A, tilde
''	=> '&Auml;',	# capital A, dieresis or umlaut mark
''	=> '&Ccedil;',	# capital C, cedilla
''	=> '&ETH;',	# capital Eth, Icelandic
''	=> '&Eacute;',	# capital E, acute accent
''	=> '&Ecirc;',	# capital E, circumflex accent
''	=> '&Egrave;',	# capital E, grave accent
''	=> '&Euml;',	# capital E, dieresis or umlaut mark
''	=> '&Iacute;',	# capital I, acute accent
''	=> '&Icirc;',	# capital I, circumflex accent
''	=> '&Igrave;',	# capital I, grave accent
''	=> '&Iuml;',	# capital I, dieresis or umlaut mark
''	=> '&Ntilde;',	# capital N, tilde
''	=> '&Oacute;',	# capital O, acute accent
''	=> '&Ocirc;',	# capital O, circumflex accent
''	=> '&Ograve;',	# capital O, grave accent
''	=> '&Oslash;',	# capital O, slash
''	=> '&Otilde;',	# capital O, tilde
''	=> '&Ouml;',	# capital O, dieresis or umlaut mark
''	=> '&THORN;',	# capital THORN, Icelandic
''	=> '&Uacute;',	# capital U, acute accent
''	=> '&Ucirc;',	# capital U, circumflex accent
''	=> '&Ugrave;',	# capital U, grave accent
''	=> '&Uuml;',	# capital U, dieresis or umlaut mark
''	=> '&Yacute;',	# capital Y, acute accent
''	=> '&aacute;',	# small a, acute accent
''	=> '&acirc;',	# small a, circumflex accent
''	=> '&aelig;',	# small ae diphthong (ligature)
''	=> '&agrave;',	# small a, grave accent
''	=> '&aring;',	# small a, ring
''	=> '&atilde;',	# small a, tilde
''	=> '&auml;',	# small a, dieresis or umlaut mark
''	=> '&ccedil;',	# small c, cedilla
''	=> '&eacute;',	# small e, acute accent
''	=> '&ecirc;',	# small e, circumflex accent
''	=> '&egrave;',	# small e, grave accent
''	=> '&eth;',	# small eth, Icelandic
''	=> '&euml;',	# small e, dieresis or umlaut mark
''	=> '&iacute;',	# small i, acute accent
''	=> '&icirc;',	# small i, circumflex accent
''	=> '&igrave;',	# small i, grave accent
''	=> '&iuml;',	# small i, dieresis or umlaut mark
''	=> '&ntilde;',	# small n, tilde
''	=> '&oacute;',	# small o, acute accent
''	=> '&ocirc;',	# small o, circumflex accent
''	=> '&ograve;',	# small o, grave accent
''	=> '&oslash;',	# small o, slash
''	=> '&otilde;',	# small o, tilde
''	=> '&ouml;',	# small o, dieresis or umlaut mark
''	=> '&szlig;',	# small sharp s, German (sz ligature)
''	=> '&thorn;',	# small thorn, Icelandic
''	=> '&uacute;',	# small u, acute accent
''	=> '&ucirc;',	# small u, circumflex accent
''	=> '&ugrave;',	# small u, grave accent
''	=> '&uuml;',	# small u, dieresis or umlaut mark
''	=> '&yacute;',	# small y, acute accent
''	=> '&yuml;',	# small y, dieresis or umlaut mark


''	=> '&copy;',	# copyright sign
''	=> '&reg;',	# registered sign
"\240" => '&nbsp;',	# non breaking space


''	=> '&iexcl;',
''	=> '&cent;',
''	=> '&pound;',
''	=> '&curren;',
''	=> '&yen;',
''	=> '&brvbar;',
''	=> '&sect;',
''	=> '&uml;',
''	=> '&ordf;',
''	=> '&laquo;',
''	=> '&not;',
''	=> '&shy;',
''	=> '&macr;',
''	=> '&deg;',
''	=> '&plusmn;',
''	=> '&sup1;',
''	=> '&sup2;',
''	=> '&sup3;',
''	=> '&acute;',
''	=> '&micro;',
''	=> '&para;',
''	=> '&middot;',
''	=> '&cedil;',
''	=> '&ordm;',
''	=> '&raquo;',
''	=> '&frac14;',
''	=> '&frac12;',
''	=> '&frac34;',
''	=> '&iquest;',
''	=> '&times ;',
''	=> '&divide;',
);

my %FORMATS = (
L	=>  [ align => 'left' ],
C	=>  [ align => 'center' ],
R	=>  [ align => 'right' ],
J	=>  [ align => 'justify' ],
T	=>  [ valign => 'top' ],
M	=>  [ valign => 'middle' ],
B	=>  [ valign => 'bottom' ],
N	=>  [ nowrap => '' ],
);

my %MAPS;

my @MAP_KEYS;


my @FILE_DIRS;			# [0] = current
my @INDENT_LVLS;		# [0] = current




sub HTML_map(@)
{
my @map_refs = @_;	#   [$win_dir, $unix_dir ]...

foreach my $ref (@map_refs)
{
my ($win_dir, $unix_dir) = @{$ref};

$win_dir =~ s!\\!/!g;


if (ENV_is_win32())
{
$MAPS{$win_dir} = $unix_dir;
push @MAP_KEYS, $win_dir;
} else
{
$MAPS{$unix_dir} = $win_dir;
push @MAP_KEYS, $unix_dir;
}
}
map { ENV_debug( "MAP $_ -> $MAPS{$_}"); } @MAP_KEYS;
}




sub HTML_doc_s($$;@)
{
my ($document_file_spec,
$title,
%attrs,		    # css, cssprint, style, script, favicon
) = @_;
my @lines;

unshift @FILE_DIRS, ENV_split_spec_d( ENV_abs_path( undef, $document_file_spec));
unshift @INDENT_LVLS, 0;


assert( \%attrs, \%DOC);

push @lines, '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">';
push @lines, "<!-- $document_file_spec -->";
push @lines, '<HTML lang="en">';
push @lines, '<HEAD>';
$INDENT_LVLS[0]++;
push @lines, indent( '<TITLE>' . HTML_txt( $title) . '</TITLE>');
if ($attrs{css})
{
my $css_file = $attrs{css};
my $html = '<LINK REL="stylesheet" HREF="{url}" TYPE="text/css">';
$html = map_url( $css_file, $html);
push @lines, indent( $html);
}
if ($attrs{cssprint})
{
my $css_file = $attrs{cssprint};
my $html = '<LINK REL="stylesheet" HREF="{url}" TYPE="text/css" media="print">';
$html = map_url( $css_file, $html);
push @lines, indent( $html);
}
if ($attrs{style})
{
push @lines, indent( '<STYLE type="text/css">');
$INDENT_LVLS[0]++;
map { push @lines, indent( $_) } split( /\s*\n\s*/, $attrs{style});
$INDENT_LVLS[0]--;
push @lines, indent( '</STYLE>');
}
if ($attrs{script})
{
my $script_file = $attrs{script};
my $html = '<SCRIPT SRC="{url}"></SCRIPT>';
$html = map_url( $script_file, $html);
push @lines, indent( $html);
}
if ($attrs{favicon})
{
my $favicon_file = $attrs{favicon};
my $html = '<LINK REL="shortcut icon" HREF="{url}" TYPE="image/x-icon">';
$html = map_url( $favicon_file, $html);
push @lines, indent( $html);
}






push @lines, indent( '<META name="SKYPE_TOOLBAR" content="SKYPE_TOOLBAR_PARSER_COMPATIBLE">');

$INDENT_LVLS[0]--;
push @lines, indent( '</HEAD>');

return @lines;
}




sub HTML_doc_e()
{
shift @FILE_DIRS;
shift @INDENT_LVLS;

return '</HTML>';
}





sub HTML_body_s(@)
{
my $attrs_ref = (defined $_[0] && ref($_[0]) eq "HASH") ? shift : {};

my @attrs = 'BODY';
push @attrs, assert( $attrs_ref, \%BODY);

return "<@attrs>";
}




sub HTML_body_e()
{
return '</BODY>';
}




sub HTML_anchor($$)
{
my ($anchor,
$text,
) = @_;

my $html = '<A NAME="' . fix_name( $anchor) . '">' . $text . '</A>';

return $html;
}





sub HTML_llink($$)
{
my ($anchor,
$text,
) = @_;

my $html = '<A HREF="#' . fix_name( $anchor) . '">' . $text . '</A>';

return $html;
}





sub HTML_link($$;$)
{
my $attrs_ref = (defined $_[0] && ref($_[0]) eq "HASH") ? shift : {};
my $url = shift;
my $text = (defined $_[0]) ? shift : '';

my @attrs = '<A HREF="{url}"';
push @attrs, assert( $attrs_ref, \%A);

return map_url( $url, "@attrs>$text</A>");
}




sub HTML_img($;@)
{
my ($url,
%attrs,
) = @_;

my @attrs = 'IMG src="{url}"';
push @attrs, assert( \%attrs, \%IMG);

my $html = "<@attrs>";

return map_url( $url, $html);
}




sub HTML_hr(@)
{
my %attrs = @_;

my @attrs = 'HR';
push @attrs, assert( \%attrs, \%HR);

return "<@attrs>";
}




sub HTML_table($$$;@)
{
my ($formats_ref,	    # refer to FORMATS
$heads_ref,
$row_refs_ref,
@attrs,
) = @_;
my @lines;

my @formats = (defined $formats_ref) ? @{$formats_ref} : ();
my @heads = (defined $heads_ref) ? @{$heads_ref} : ();
$row_refs_ref = [ [] ] if (!defined $row_refs_ref || !defined $row_refs_ref->[0]);

my $nr_formats = @formats;
my $nr_heads = @heads;

my $nr_cols = $nr_formats;
$nr_cols = $nr_heads
if ($nr_heads > $nr_cols);
$nr_cols = @{$row_refs_ref->[0]}
if (@{$row_refs_ref->[0]} > $nr_cols);





my @row_formats;
if (@formats && grep( /^-/, @formats) == @formats)
{
foreach my $f (split( '', substr( $formats[0], 1)))
{
if (grep( /$f/, @formats) == scalar @formats)
{

push @row_formats, @{$FORMATS{$f}};
map { $_ =~ s/$f// } @formats;
}
}
}




for (my $i = 0; $i < $nr_cols; $i++)
{
my $format = $formats[$i];
if (defined $format)
{
if ($format eq '')
{
$format = {};
} elsif (substr( $format, 0, 1) eq '-')
{
my @attrs;
foreach my $f (split( '', substr($format, 1)))
{

push @attrs, @{$FORMATS{$f}};
}
$format = { @attrs };
} else
{
$format = { $format };
}
$formats[$i] = $format;
} else
{
$formats[$i] = {};
}
}

push @lines, HTML_table_s( @attrs);




if (defined $heads_ref)
{
push @lines, HTML_tr( @row_formats);
push @lines, table_rows( $heads_ref, [ @formats ], $nr_cols, \&HTML_th);
}




foreach my $row_ref (@{$row_refs_ref})
{

push @lines, HTML_tr( @row_formats);
push @lines, table_rows( $row_ref, [ @formats ], $nr_cols, \&HTML_td);
}
push @lines, HTML_table_e();


return (wantarray) ? @lines : "@lines";
}




sub table_rows($$$$)
{
my ($row_ref,
$format_refs_ref,
$nr_cols,
$cell_func,
) = @_;
my @lines;

my $i = 0;
do
{
my $cell = $row_ref->[$i];
my $format_ref = $format_refs_ref->[$i];

my %format = %{$format_ref};
$i++;
my $colspan = 1;
while ($i < $nr_cols && !defined $row_ref->[$i])
{
$i++;
$colspan++;
}

$cell = '&nbsp;' if (!defined $cell);
if ($colspan > 1)
{
$format{colspan} = $colspan;
}
if (ref $cell)
{
my $cell_format_ref = $cell->[0];
map { $format{$_} = $cell_format_ref->{$_} } keys %{$cell_format_ref};
$cell = $cell->[1];
}
push @lines, &$cell_func( \%format, $cell);
} while ($i < $nr_cols);

return @lines;
}




sub HTML_table_s(@)
{
my %attrs = @_;
my $html;

my @attrs = 'TABLE';
push @attrs, assert( \%attrs, \%TABLE);

$html = indent( "<@attrs>");
$INDENT_LVLS[0] += 3;

return $html;
}




sub HTML_table_e()
{
$INDENT_LVLS[0] -= 3;

return indent( '</TABLE>');
}




sub HTML_tr(@)
{
my %attrs = @_;
my $html;

my @attrs = 'TR';
push @attrs, assert( \%attrs, \%TR);

$INDENT_LVLS[0] -= 2;
$html = indent( "<@attrs>");
$INDENT_LVLS[0] += 2;

return $html;
}





sub HTML_trth(@)
{
my $attrs_ref = (defined $_[0] && ref($_[0]) eq "HASH") ? shift : {};
my $html;

my @attrs = 'TR><TH';
push @attrs, assert( $attrs_ref, \%THTD);

my $content;
if (@_)
{
$content = join( '', @_);
$content = '&nbsp;' if ($content eq '');
} else
{
$content = '';
}

$INDENT_LVLS[0] -= 2;
$html = indent( "<@attrs>$content");
$INDENT_LVLS[0] += 2;

return $html;
}





sub HTML_trtd(@)
{
my $attrs_ref = (defined $_[0] && ref($_[0]) eq "HASH") ? shift : {};
my $html;

my @attrs = 'TR><TD';
push @attrs, assert( $attrs_ref, \%THTD);

my $content;
if (@_)
{
$content = join( '', @_);
$content = '&nbsp;' if ($content eq '');
} else
{
$content = '';
}

$INDENT_LVLS[0] -= 2;
$html = indent( "<@attrs>$content");
$INDENT_LVLS[0] += 2;

return $html;
}





sub HTML_th(@)
{
my $attrs_ref = (defined $_[0] && ref($_[0]) eq "HASH") ? shift : {};
my $html;

my @attrs = 'TH';
push @attrs, assert( $attrs_ref, \%THTD);

my $content;
if (@_)
{
$content = join( '', @_);
$content = '&nbsp;' if ($content eq '');
} else
{
$content = '';
}

$INDENT_LVLS[0]--;
$html = indent( "<@attrs>$content");
$INDENT_LVLS[0]++;

return $html;
}





sub HTML_td(@)
{
my $attrs_ref = (defined $_[0] && ref($_[0]) eq "HASH") ? shift : {};
my $html;

my @attrs = 'TD';
push @attrs, assert( $attrs_ref, \%THTD);

my $content;
if (@_)
{
$content = join( '', @_);
$content = '&nbsp;' if ($content eq '');
} else
{
$content = '';
}

$INDENT_LVLS[0]--;
$html = indent( "<@attrs>$content");
$INDENT_LVLS[0]++;

return $html;
}




sub HTML_br(@)
{
my %attrs = @_;

my @attrs = 'BR';
push @attrs, assert( \%attrs, \%BR);

return "<@attrs>";
}





sub HTML_ul(@)
{
return generic_align_4( 'UL', @_);
}





sub HTML_ol(@)
{
return generic_align_4( 'OL', @_);
}





sub HTML_li(@)
{
my $html;

$INDENT_LVLS[0]++;
$html = "\n" . indent( generic_align_4( 'LI', @_));
$INDENT_LVLS[0]--;

return $html;
}





sub HTML_p(@)
{
return generic_align_4( 'P', @_);
}





sub HTML_h($@)
{
my $level = shift;

return generic_align_4( "H$level", @_);
}





sub HTML_div(@)
{
return generic_align_4( 'DIV', @_);
}





sub HTML_span(@)
{
my $attrs_ref = (defined $_[0] && ref($_[0]) eq "HASH") ? shift : {};

my @attrs = 'SPAN';
push @attrs, assert( $attrs_ref, \%GENERIC);

my $content = join( '', @_);
$content = '&nbsp;' if ($content eq '');

return "<@attrs>$content</SPAN>";
}





sub generic_align_4($@)
{
my $tag = shift;
my $attrs_ref = (defined $_[0] && ref($_[0]) eq "HASH") ? shift : {};

my @attrs = "$tag";
push @attrs, assert( $attrs_ref, \%GENERIC_ALIGN_4);

my $content = join( '', @_);

$content = '&nbsp;' if ($content eq '');

return "<@attrs>$content\n</$tag>";
}




sub HTML_bold(@)
{
my $text = join( '', @_);

return "<B>$text</B>";
}




sub HTML_bold_txt(@)
{
my $text = HTML_txt( @_);

return "<B>$text</B>";
}




sub HTML_italic(@)
{
my $text = join( '', @_);

return "<I>$text</I>";
}




sub HTML_italic_txt(@)
{
my $text = HTML_txt( @_);

return "<I>$text</I>";
}




sub HTML_txt(@)
{
my $text = join( '', @_);

$text =~ s/([^\t\n !#\$%(-;=?-~])/$HTML_CHARS{$1} || sprintf '&#x%X;', ord($1)/ge;

$text =~ s/(\n)/<BR>/g;			    # replace \n by <br>
$text =~ s/( {2,})/'&nbsp;' x length $1/ge;	    # replace 2 and more spaces by &nbsp;
$text =~ s/^ /&nbsp;/;			    # replace single leading space by &nbsp;
$text =~ s/ $/&nbsp;/;			    # replace single trailing space by &nbsp;

return $text;
}




sub HTML_pre_s(@)
{
my %attrs = @_;

$attrs{style} = "margin-top:0;margin-bottom:0"
if (!$attrs{style});

my @attrs = 'PRE';
push @attrs, assert( \%attrs, \%GENERIC);

return "<@attrs>";
}




sub HTML_pre_txt(@)
{
my $text = join( '', @_);

$text =~ s/([^\t\n !#\$%(-;=?-~])/$HTML_CHARS{$1} || sprintf '&#x%X;', ord($1)/ge;


return $text;
}




sub HTML_pre_e()
{
return '</PRE>';
}




sub assert($$)
{
my ($i_ref,
$def_ref,
) = @_;
my @attrs;

my @def_keys = keys %{$def_ref};
foreach my $key (keys %{$i_ref})
{
ENV_fatal( "No such key '$key' (@def_keys)")
if (!grep $key eq $_, @def_keys);
my $value = $i_ref->{$key};
my $def_key = $def_ref->{$key};
$def_key = $key if ($def_key eq '');

my $def_value = $TYPES{$def_key};
ENV_fatal( "No TYPE for key '$def_key'")
if (!defined $def_value);
if (ref $def_value)
{
my @def_values = @{$def_value};

ENV_fatal( "'$key=$value' Invalid value (@def_values)")
if (!grep $value eq $_, @def_values);
push @attrs, "$key=\"$value\"";
} else
{
if ($def_value eq '')
{
ENV_fatal( "'$key=$value' No value assignent allowed")
if (defined $value && $value ne '' && $value ne '1');
push @attrs, $key;
} else
{
if ($def_value eq 't')
{
$value = HTML_txt( $value);
} elsif ($def_value eq 'l')
{
ENV_fatal( "$key=>$value must be number (%)")
if ( $value !~ /^\d+%?$/);
} elsif ($def_value eq 'n')
{
ENV_fatal( "$key=>$value must be number")
if ( $value !~ /^\d+$/);
} elsif ($def_value eq 'u')
{
$value =~ s/ /%20/g;
} elsif ($def_value eq 'c')
{
;
} elsif ($def_value eq 'i')
{
;
} elsif ($def_value eq 's')
{
ENV_fatal( "$key=>$value must be single char")
if (length $value > 1);
} else
{
ENV_fatal( "Unknown TYPE '$def_value' for '$key'");
}
push @attrs, "$key=\"$value\"";
}
}
}
return @attrs;
}




sub map_url($$)
{
my ($url,
$html,	    # contains '{url}' where the converted $url must be inserted
) = @_;


$url =~ s/ /%20/g;			# spaces


if ($url =~ m!^http://!)
{




$html =~ s/{url}/$url/;
} else
{
$url = ENV_perl_path_nospace( $url);


if (ENV_is_abs_path_perl( $url))
{



$url = ENV_rel_path( $FILE_DIRS[0], $url);

}
if (ENV_is_abs_path_perl( $url))
{



if (@MAP_KEYS)
{



my $win_url = $url;
my $unix_url = $url;
my $this_url = $url;




my $found_key;
my $remainder;
foreach my $key (@MAP_KEYS)
{
if ($url =~ /^$key(.*)/)
{
$found_key = "$key";
$remainder = $1;
last;
}
}

if (defined $found_key)
{

my $other_url = "$MAPS{$found_key}$remainder";
if (ENV_is_win32())
{
$unix_url = $other_url;


} else
{
$win_url = $other_url;

}


(my $win_html = $html) =~ s!{url}!file://$win_url!g;
(my $unix_html = $html) =~ s!{url}!file://$unix_url!g;
(my $this_html = $html) =~ s!{url}!file://$this_url!g;

$html = "<SCRIPT type=\"text/javascript\" LANGUAGE=\"JavaScript\"> <!--\n".
"	 if (navigator.userAgent.indexOf('Win') != -1)\n" .
"	   document.write('$win_html');\n" .
"	 else\n" .
"	   document.write('$unix_html');\n" .
"	 //-->\n" .
"  </SCRIPT>\n" .
"  <NOSCRIPT>$this_html</NOSCRIPT>\n";
} else
{
ENV_error( "No MAPping found for '$url' in (@MAP_KEYS)\n");
$url = "file://$url";
$html =~ s/{url}/$url/;
}

} else
{



$url = "file://$url";
$html =~ s/{url}/$url/;
}
} else
{





$html =~ s/{url}/$url/;
}
}



return $html;
}




sub fix_name($)
{
my ($name) = @_;

$name =~ s/ /%20/g;

return $name;
}




sub indent($)
{
my ($cell) = @_;

$INDENT_LVLS[0] = 0
if (!defined $INDENT_LVLS[0]);
my $indent = '  ' x $INDENT_LVLS[0];
$cell =~ s/\n/\n$indent/g;

return "$indent$cell";
}

return 1;
