#===============================================================
#
#   notify.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package notify;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
NOTIFY_submit
NOTIFY_get_lines
NOTIFY_get_xterm_data
);

use strict;
use integer;

use glo::env;
use glo::proc;
use glo::banner;
use gbsenv;




sub NOTIFY_submit($$$$);
sub NOTIFY_get_lines($$$$);
sub NOTIFY_get_xterm_data($$$);








sub NOTIFY_submit($$$$)
{
my ($job_name,
$rc,
$logfile,
$state,
) = @_;

ENV_say( 1, "Notify user (if present)");
if (GBSENV_gui_available())
{
PROC_spawn_detached( ENV_command_ref( 'notify_tkx.pl', [ $job_name, $rc, $logfile, $state ]));
} else
{
my @lines = NOTIFY_get_lines( $job_name, $rc, $logfile, $state);
my ($title, $fg, $bg, $cols, $rows, $icon) = NOTIFY_get_xterm_data( $job_name, $rc, \@lines);
PROC_spawn_xterm( ENV_command_ref( 'notify.pl', [ $job_name, $rc, $logfile, $state ]),
$title, $fg, $bg, $cols, $rows, $icon);
}
}




sub NOTIFY_get_lines($$$$)
{
my ($job_name,
$rc,
$logfile_spec,
$state,
) = @_;
my @lines;

push @lines, "Batch Job  : $job_name $state-END",
"Exit-Status: $rc",
"Logfile    : $logfile_spec";

if ($rc == 0)
{
@lines = BANNER_box( '-', '| ', \@lines);
} else
{
@lines = BANNER_box( '#', '## ', \@lines);
}



return @lines;
}




sub NOTIFY_get_xterm_data($$$)
{
my ($job_name,
$rc,
$lines_ref       # Must be result of NOTIFY_get_lines
) = @_;
my ($title, $fg, $bg, $cols, $rows, $icon);

$title = "GBS: Notify $job_name";
if ($rc == 0)
{
$fg = 'black';
$icon = "$ENV{GBS_SCRIPTS_PATH}/doc/images/gbs.gif";
} else
{
$fg = 'bright_red';
$icon = "$ENV{GBS_SCRIPTS_PATH}/doc/images/gbs_test.gif";
}
$bg = 'white';

$cols = length($lines_ref->[0]) + 1;    # \n
$rows = @{$lines_ref} + 2;		    # 1 ASK + 1 extra

return ($title, $fg, $bg, $cols, $rows, $icon);
}

return 1;


