#=================================================
#
#   plugin.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package plugin;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(
PLUGIN_new_target
PLUGIN_remove_target
PLUGIN_dialog
);
use strict;
use integer;

use glo::env;
use glo::list;
use glo::ask;
use glo::slurp;
use glo::file;
use glo::format;
use glo::scm;
use gbsenv;
use gbsglo;
use fspec;
use fcreate;
use files;
use swt;
use swa;
use gbsscm;
use system;
use plg;
use run;




sub PLUGIN_new_target($);
sub PLUGIN_remove_target($);
sub PLUGIN_dialog($$);

sub main_remove_att($$$);
sub main_update_dialog($$$);
sub main_new_att($$$);
sub remove_att($$);
sub update_dialog($$);
sub update_plugin($$$);
sub new_build($$);
sub build_create_dirs($$$$);
sub build_update_dirs($$$);
sub new_audit($$);
sub audit_create_dirs($$$$$);
sub audit_update_dirs($$$);
sub new_tool($$);
sub tools_create_dirs($$);
sub tools_update_dirs($$$);
sub select_new_att($);
sub select_att($$);
sub ask_new_data($$);
sub select_plugin_name();
sub select_org_att();
sub do_install($$$);
sub get_plugins();
sub do_plugin_file($$$);
sub copy_1_file($$$$);
sub select_selector_atts($$$);
sub add_dirs($);
sub system_add($$);
sub system_store();
sub setup_type($);
sub setup_plugin($);
sub setup_att($);
sub do_gbsext($);




my $IS_WIN32 = ENV_is_win32();
my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();
my $GBS_ROOT_PATH = GBSENV_gbs_root_path();

my $DEFAULT_GEN_NAME = ($IS_WIN32) ? 'mvs' : 'linux';











my %PLUGIN_SETUP = (

audit   => [ qw( audit Audit  audit.gbs  audits)	 ],
build   => [ qw( build Target target.gbs targets)    ],	# We want platform Targets only
tools   => [ qw( tools Tool   tool.gbs   tools)      ],
);

my %EXCLUDE_ATT_NAMES = (

audit   => [ qw (ALL) ],
build   => [ qw (ALL makemake_stubs) ],
tools   => [ qw (ALL) ],
);




my $ATT_TYPE;		    # 'audit',     'build',      'tools'
my $ATT_TYPE_NAME;	    # 'Audit',     'Target',     'Tool'
my $ATT_FILE;		    # 'audit.gbs', 'target.gbs', 'tool.gbs'
my $ATT_TYPE_ID;	    # 'audits',    'targets',    'tools'	# also used for $ATTS_ENV_NAME

my $ATT_TYPE_NAME_S;		# 'Audits',     'Targets',     'Tools'

my $ATT_ENV_NAME;		# 'GBS_AUDIT', 'GBS_TARGET', 'GBS_TOOL'
my $ATTS_ENV_NAME;		# 'GBS_AUDITS', 'GBS_TARGETS', 'GBS_TOOLS'
my $ALL_ATTS_ENV_NAME;		# 'GBS_ALL_AUDITS', 'GBS_ALL_TARGETS', 'GBS_ALL_TOOLS'

my $PLUGIN_NAME;		# e.g.: mingw_cpp
my $GBS_SCRIPTS_PLUGINS_PATH;	# $GBS_SCRIPTS_PATH/plugins/$ATT_TYPE
my $GBS_SCRIPTS_PLUGIN_PATH;	# $GBS_SCRIPTS_PATH/plugins/$ATT_TYPE/$PLUGIN_NAME

my $ATT_NAME;			# e.g.: $GBS_TARGET
my $GBS_ATTS_PATH;		# $GBS_ROOT_PATH/sys$ATT_TYPE
my $GBS_ATT_PATH;		# $GBS_ROOT_PATH/sys$ATT_TYPE/$ATT_NAME

my $GBS_ATT_VALUE;
my @GBS_ATT_VALUES;
my @GBS_ALL_ATT_VALUES;

my $ACTION;	# P_REF P_ORG O_NEW O_REF or COPY




my %PLUGINS;

my $PLUGINS_MAX_NAME;




sub PLUGIN_new_target($)
{
my ($default_target) = @_;

return new_build( 'build', $default_target);
}




sub PLUGIN_remove_target($)
{
my ($default_target) = @_;

return remove_att( 'build', $default_target);
}




sub PLUGIN_dialog($$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

ENV_say( 1, "GBS Plugins");

GBSSCM_preset( 0);

my @menu_items = (


[ 'Build: Update',		\&main_update_dialog, 'build' ],

[ "\nAudit: Define/Install",	\&main_new_att,      'audit' ],
[ 'Audit: Update',		\&main_update_dialog, 'audit' ],
[ 'Audit: Remove',		\&main_remove_att,   'audit' ],
[ 'Tools: Define/Install',	\&main_new_att,      'tools' ],
[ 'Tools: Update',		\&main_update_dialog, 'tools' ],
[ 'Tools: Remove',		\&main_remove_att,   'tools' ],
);

ASK_menu( 'Select function to perform:', \@menu_items, $entries_ref);
}




sub main_remove_att($$$)
{
my ($menu_entry,
$data_ref,		    # $att_type
$entries_ref,
) = @_;

my $att_type = $data_ref;
GBSSCM_connect();

remove_att( $att_type, '');	# no default_att
}




sub main_update_dialog($$$)
{
my ($menu_entry,
$data_ref,		    # $att_type
$entries_ref,
) = @_;

my $att_type = $data_ref;
GBSSCM_connect();

update_dialog( $att_type, '');	# no default_att
}




sub main_new_att($$$)
{
my ($menu_entry,
$data_ref,		    # $att_type
$entries_ref,
) = @_;

my $att_type = $data_ref;
GBSSCM_connect();

if ($att_type eq 'build')
{
new_build( $att_type, '');
} elsif ($att_type eq 'audit')
{
new_audit( $att_type, '');
} elsif ($att_type eq 'tools')
{
new_tool( $att_type, '');
} else
{
ENV_fatal( "Impossible att_type: $att_type");
}
}




sub remove_att($$)
{
my ($att_type,		# audit build tools
$default_att,
) = @_;
my $removed_att = '';	# '' if nothing removed

setup_type( $att_type);
ENV_say( 0, '');
ENV_say( 1, "Remove $ATT_TYPE_NAME $default_att");

ENV_say( 1, "Current $ATT_TYPE_NAME_S = '@GBS_ATT_VALUES'");

my $att_to_remove = select_att( remove => $default_att);
if ($att_to_remove ne '')
{



setup_att( $att_to_remove);	# Set $ATT_NAME and GBS_ATT_PATH

SYSTEM_remove_os_ref( $GBS_ROOT_PATH, '.', $ATT_TYPE_ID => $att_to_remove);
@GBS_ATT_VALUES = grep( $att_to_remove ne $_, @GBS_ATT_VALUES);
system_store();

ENV_say( 1, "$ATT_TYPE_NAME $att_to_remove removed",
"Note that the $ATT_TYPE_NAME Directories/Files are NOT deleted");
ENV_say( 1, "Remaining $ATT_TYPE_NAME_S: '@GBS_ATT_VALUES'");

$removed_att = $att_to_remove;
setup_att( undef);	# Clear $ATT_NAME and GBS_ATT_PATH
}

return $removed_att;
}




sub update_dialog($$)
{
my ($att_type,		# audit build tools
$default_att,
) = @_;

setup_type( $att_type);
ENV_say( 0, '');
ENV_say( 1, "Update $ATT_TYPE_NAME $default_att");




my ($att_name, $plugin_name) = select_att( update => '');
if ($att_name ne '')
{



setup_att( $att_name);    # Set $ATT_NAME and GBS_ATT_PATH

my $update_dirs_func;
if ($att_type eq 'audit')
{
$update_dirs_func = \&audit_update_dirs;
} elsif ($att_type eq 'build')
{
$update_dirs_func = \&build_update_dirs;
} else #($att_type eq 'tools')
{
$update_dirs_func = \&tools_update_dirs;
}

my @menu_items = (
[ 'Add/Repair Directories',
$update_dirs_func, $att_name ],
[ "Update $ATT_TYPE_NAME $att_name [$plugin_name]",
\&update_plugin, [ $att_name, $plugin_name ] ],
);
ASK_menu( "Select Update for $att_name", \@menu_items, undef);

}
}




sub update_plugin($$$)
{
my ($menu_entry,
$data_ref,		    # [ $att_name, $plugin_name ]
$entries_ref) = @_;

my ($att_name, $plugin_name) = @{$data_ref};

ENV_say( 1, "Update $ATT_TYPE_NAME $att_name [$plugin_name]");

$plugin_name = select_plugin_name()
if (!$plugin_name);

do_plugin_file( $plugin_name, $att_name, 'U')
if ($plugin_name);
}




sub new_build($$)
{
my ($att_type,		# audit build tools
$default_att,
) = @_;
my $new_target;

setup_type( $att_type);
ENV_say( 0, '');
ENV_say( 1, "New $ATT_TYPE_NAME $default_att");

my $default_target = $default_att;	    #TBR

$new_target = select_new_att( $default_target);




if ($new_target ne '')
{




setup_att( $new_target);   # Set $ATT_NAME and GBS_ATT_PATH
my ($plugin_name,
$org_att,
undef,	    # $targets_ref
$subsys_ref,
$comp_selector,
$components_sel_ref) = ask_new_data( $new_target, 1);




ENV_say( 1, "Creating Target '$new_target'...");




build_create_dirs( $new_target, $subsys_ref, $comp_selector, $components_sel_ref);




do_install( $plugin_name, $org_att, $new_target);




system_add( $new_target, $plugin_name);
SWT_set( $new_target, 0);

ENV_say( 1, "Target '$new_target' created");




$ACTION = undef;




do_gbsext( $plugin_name);
} else
{
ENV_warn( "No new Target selected!");
}

return $new_target;
}




sub build_create_dirs($$$$)
{
my ($target,
$subsys_ref,
$comps_selector,
$components_sel_ref,
) = @_;

my @new_dirs;
push @new_dirs, "$GBS_ROOT_PATH/sysbuild/$target";
push @new_dirs, "$GBS_ROOT_PATH/sysbuild/makemake_stubs/$target";
push @new_dirs, "$GBS_ROOT_PATH/tmp/$target";

my @subsystems = @{$subsys_ref};
foreach my $subsys (@subsystems)
{
push @new_dirs, "$GBS_ROOT_PATH/dev/$subsys/build/$target";

if (GBSGLO_subsystem_is_full_gbs( $subsys))
{
my @components = select_selector_atts( $comps_selector, $components_sel_ref, [ GBSGLO_components( $subsys) ]);
foreach my $component (@components)
{
push @new_dirs, "$GBS_ROOT_PATH/dev/$subsys/comp/$component/bld/$target";
}
}
}

ENV_setenv( GBS_ALL_TARGETS => [ ENV_getenv( 'GBS_ALL_TARGETS'), $target ]);
FILES_reset( undef);
add_dirs( \@new_dirs);
}




sub build_update_dirs($$$)
{
my ($menu_entry,
$data_ref,		    # $target
$entries_ref) = @_;

my $target = $data_ref;

ENV_say( 1, "Add/Repair Dirs for Target $target");

my (undef,			# $plugin_name,
undef,			# $org_att
undef,			# $targets_ref
$subsys_ref,
$comp_selector,
$components_sel_ref) = ask_new_data( $target, 0);

ENV_say( 1, "Updating Target '$target'...");
build_create_dirs( $target, $subsys_ref, $comp_selector, $components_sel_ref);
}




sub new_audit($$)
{
my ($att_type,		# audit build tools
$default_att,
) = @_;
my $new_audit;

setup_type( $att_type);
ENV_say( 0, '');
ENV_say( 1, "New $ATT_TYPE_NAME $default_att");

$new_audit = select_new_att( '');




if ($new_audit ne '')
{
setup_att( $new_audit);   # Set $ATT_NAME and GBS_ATT_PATH
my ($plugin_name,
$org_att,
$targets_ref,
$subsys_ref,
$comp_selector,
$components_sel_ref) = ask_new_data( $new_audit, 1);




ENV_say( 1, "Creating Audit '$new_audit'...");




audit_create_dirs( $new_audit, $targets_ref, $subsys_ref, $comp_selector, $components_sel_ref);




do_install( $plugin_name, $org_att, $new_audit);




system_add( $new_audit, $plugin_name);
SWA_set( $new_audit, 0);

ENV_say( 1, "Audit '$new_audit' created");




$ACTION = undef;




do_gbsext( $plugin_name);
} else
{
ENV_warn( "No new Audit selected!");
}

return $new_audit;
}




sub audit_create_dirs($$$$$)
{
my ($audit,
$targets_ref,
$subsys_ref,
$comps_selector,
$components_sel_ref,
) = @_;

my @new_dirs;
push @new_dirs, "$GBS_ROOT_PATH/sysaudit/$audit";
push @new_dirs, "$GBS_ROOT_PATH/tmp/$audit";

my @targets = @{$targets_ref};
foreach my $target (@targets)
{
my @subsystems = @{$subsys_ref};
foreach my $subsys (@subsystems)
{
push @new_dirs, "$GBS_ROOT_PATH/dev/$subsys/audit/$audit";
push @new_dirs, "$GBS_ROOT_PATH/dev/$subsys/audit/$audit/$target";

if (GBSGLO_subsystem_is_full_gbs( $subsys))
{
my @components = select_selector_atts( $comps_selector, $components_sel_ref, [ GBSGLO_components( $subsys) ]);
foreach my $component (@components)
{
push @new_dirs, "$GBS_ROOT_PATH/dev/$subsys/comp/$component/aud/$audit";
push @new_dirs, "$GBS_ROOT_PATH/dev/$subsys/comp/$component/aud/$audit/$target";
}
}
}
}

ENV_setenv( GBS_ALL_AUDITS => [ ENV_getenv( 'GBS_ALL_AUDITS'), $audit ]);
FILES_reset( undef);
add_dirs( \@new_dirs);
}




sub audit_update_dirs($$$)
{
my ($menu_entry,
$data_ref,		    # $audit
$entries_ref) = @_;

my $audit = $data_ref;

ENV_say( 1, "Add/Repair Dirs for Audit $audit");

my (undef,			    # $plugin_name
undef,			    # $org_or_att
$targets_ref,
$subsys_ref,
$comp_selector,
$components_sel_ref) = ask_new_data( $audit, 0);

ENV_say( 1, "Updating Audit '$audit'...");
audit_create_dirs( $audit, $targets_ref, $subsys_ref, $comp_selector, $components_sel_ref);
}




sub new_tool($$)
{
my ($att_type,		# audit build tools
$default_att,
) = @_;
my $new_tool;

setup_type( $att_type);
ENV_say( 0, '');
ENV_say( 1, "New $ATT_TYPE_NAME $default_att");

$new_tool = select_new_att( '');




if ($new_tool ne '')
{
setup_att( $new_tool);   # Set $ATT_NAME and GBS_ATT_PATH
my ($plugin_name,
$org_att,
undef,	    # $targets_ref
$subsys_ref,
undef,	    # $comps_selector
undef) = ask_new_data( $new_tool, 1);




ENV_say( 1, "Creating Tool '$new_tool'...");




tools_create_dirs( $new_tool, $subsys_ref);




do_install( $plugin_name, $org_att, $new_tool);




system_add( $new_tool, $plugin_name);
GBSENV_setenv( GBS_TOOL => $new_tool);

ENV_say( 1, "Tool '$new_tool' created");




$ACTION = undef;




do_gbsext( $plugin_name);
} else
{
ENV_warn( "No new Tool selected!");
}

return $new_tool;
}




sub tools_create_dirs($$)
{
my ($tool,
$subsys_ref,
) = @_;

my @new_dirs;
push @new_dirs, "$GBS_ROOT_PATH/systools/$tool";
push @new_dirs, "$GBS_ROOT_PATH/tmp/$tool";

my @subsystems = @{$subsys_ref};
foreach my $subsys (@subsystems)
{
push @new_dirs, "$GBS_ROOT_PATH/dev/$subsys/tools/$tool";
}

ENV_setenv( GBS_ALL_TOOLS => [ ENV_getenv( 'GBS_ALL_TOOLS'), $tool ]);
FILES_reset( undef);
add_dirs( \@new_dirs);
}




sub tools_update_dirs($$$)
{
my ($menu_entry,
$data_ref,		    # $tool
$entries_ref) = @_;

my $tool = $data_ref;

ENV_say( 1, "Add/Repair Dirs for Tool $tool");

my (undef,		# $plugin_name
undef,		# org_att
undef,		# $targets_ref
$subsys_ref,
undef,		# $comps_selector
undef) = ask_new_data( $tool, 0);

ENV_say( 1, "Updating Tool '$tool'...");
tools_create_dirs( $tool, $subsys_ref);
}




sub select_att($$)
{
my ($function,	    # remove update refer copy
$wanted_att,	    # '' == force ask. Not eq '' == do not ask, take it if it exists
) = @_;
my ($att, $plugin_name) = ( '', undef);

$wanted_att = ''
if (!defined $wanted_att);
if ($wanted_att eq '')
{
if (@GBS_ATT_VALUES)
{
my @menu_items = ( [ '', '', undef ]);
foreach my $att_name (@GBS_ATT_VALUES)
{
my $plugin_name = SYSTEM_get_os_ref_value( $GBS_ROOT_PATH, '.', $ATT_TYPE_ID, $att_name);
push @menu_items, [ "$att_name [$plugin_name]", $att_name, $plugin_name ];
};
my $default_att = ($function eq 'remove') ? '' : $GBS_ATT_VALUE;

($att, $plugin_name) = ASK_value_from_menu( "Select $ATT_TYPE_NAME to $function",
\$default_att, @menu_items);
} else
{
ENV_say( 1, "No $ATT_TYPE_NAME_S defined!");
}
} else
{
if (grep $_ eq $wanted_att, @GBS_ATT_VALUES)
{
$att = $wanted_att;
$plugin_name = SYSTEM_get_os_ref_value( $GBS_ROOT_PATH, '.', $ATT_TYPE_ID, $att)
if (wantarray);
} else
{
ENV_error_exit( "$ATT_TYPE_NAME '$wanted_att' does not exists");
}
}

if ($att ne '')
{
ENV_say( 1, "Selected: $att [$plugin_name]");
} else
{
ENV_say( 1, "Selected: None");
}

return wantarray ? ($att, $plugin_name) : $att;
}




sub select_new_att($)
{
my ($default_att) = @_;
my $new_att;	    # '' if not selected

ENV_say( 1, "Create New $ATT_TYPE_NAME");

my @gbs_all_targets = SYSTEM_get_os_ref_names( $GBS_ROOT_PATH, '*', 'targets');
my @gbs_all_audits  = SYSTEM_get_os_ref_names( $GBS_ROOT_PATH, '*', 'audits');
my @gbs_all_tools   = SYSTEM_get_os_ref_names( $GBS_ROOT_PATH, '*', 'tools');

{



ENV_say( 1, "All Items:");
my @row_refs = ( [ 'All Targets', "@gbs_all_targets" ],
[ 'All_Audits',  "@gbs_all_audits" ],
[ 'ALL Tools',   "@gbs_all_tools" ],
);
ENV_say( 0, FORMAT_table( 0, 2, ': ', undef, @row_refs));
}

{



if (@GBS_ATT_VALUES)
{
ENV_say( 1, "Current $ATT_TYPE_NAME_S:");
my @atts;
foreach my $att_name (@GBS_ATT_VALUES)
{
my $plugin_name = SYSTEM_get_os_ref_value( $GBS_ROOT_PATH, '.', $ATT_TYPE_ID, $att_name);
push @atts, "$att_name [$plugin_name]";
};
ENV_say( 0, FORMAT_cols( 0, 2, '  |  ', 2, \@atts));
} else
{
ENV_say( 1, "No $ATT_TYPE_NAME_S defined");
}
}




my @exclude_names = ( @gbs_all_targets, @gbs_all_audits, @gbs_all_tools, @{$EXCLUDE_ATT_NAMES{$ATT_TYPE}} );
$default_att = ''
if (grep( $default_att eq $_, @exclude_names));
do
{
$new_att = ASK_not_word( "Enter New $ATT_TYPE_NAME-name", $default_att, \@exclude_names, 20);
if ($new_att ne '')
{
my $att_path = "$GBS_ATTS_PATH/$new_att";
if (-d $att_path)
{
my @files = SLURP_dir_all( $att_path, 0);
ENV_say( 1, "  $ATT_TYPE_NAME directory '$new_att' already exists",
"  It Contains:",
"    @files");
$new_att = undef
if (ASK_YNQ( "  Use it ($new_att) anyway?", 'N', 1) eq 'N');
}
}
} while (!defined $new_att);

return $new_att;
}




sub ask_new_data($$)
{
my ($att_name,	    # name of target, audit or tool
$ask_new,	    # bool: 1 = ask for new, 0 = ask for update
) = @_;
my ($plugin_name, $org_att, $targets_ref, $subsys_ref, $comps_selector, $components_sel_ref);
my $ask_targets = ($ATT_TYPE eq 'audit') ? 1 : 0;
my $ask_subsystems = 1;
my $ask_components = GBSGLO_subsystems_full_gbs() > 0  && ($ATT_TYPE eq 'build' || $ATT_TYPE eq 'audit');




my @gbs_targets = ENV_getenv( 'GBS_TARGETS');
my $targets_selector;
my @targets = @gbs_targets;
my @all_subsystems = GBSGLO_subsystems();
my $subsystems_selector;
my @subsystems = @all_subsystems;
my @all_components;	# cannot determine now
my $components_selector;
my @components;

my $all_ok;
do
{
$all_ok = 1;




if ($ask_new)
{
setup_plugin( undef);   # Clear $PLUGIN_NAME and $GBS_SCRIPTS_PLUGIN_PATH
my @menu_item_refs;

push @menu_item_refs, [ '<quit>', undef ];
push @menu_item_refs, [ "Create a Reference to a $ATT_TYPE_NAME Plugin", 'P_REF'  ];
push @menu_item_refs, [ "Create a Copy of a $ATT_TYPE_NAME Plugin",      'P_ORG'  ];
push @menu_item_refs, [ "Create a New own $ATT_TYPE_NAME",		'O_NEW'  ];
push @menu_item_refs, [ "Create a Reference to an own $ATT_TYPE_NAME",	'O_REF'  ];
push @menu_item_refs, [ "Create a Copy of an existing $ATT_TYPE_NAME",   'COPY'   ];
map { $_->[0] =~ s/ a A/ an A/ } @menu_item_refs;	    # $display_text

($ACTION) =
ASK_value_from_menu( "Select action for Creating new $ATT_TYPE_NAME", 1, @menu_item_refs);
if ($ACTION eq 'P_REF' || $ACTION eq 'P_ORG')
{
$plugin_name = select_plugin_name();
if (defined $plugin_name)
{
setup_plugin( $plugin_name);       # Set $PLUGIN_NAME and $GBS_SCRIPTS_PLUGIN_PATH
} else
{
$all_ok = 0;
}
} elsif ($ACTION eq 'O_NEW')
{
$plugin_name = $att_name;
} elsif ($ACTION eq 'O_REF')
{
($org_att, $plugin_name) = select_att( refer => '');  # force ask
$all_ok = 0
if ($org_att eq '');
} elsif ($ACTION eq 'COPY')
{
($org_att, $plugin_name) = select_org_att();
$all_ok = 0
if (!defined $org_att);
} else
{
ENV_fatal( "Unhandled action '$ACTION'");
}
} else
{
$ACTION = '';
}




if ($all_ok)
{
ENV_say( 0, '');
if ( $ask_targets && @gbs_targets > 1 ||
$ask_subsystems && @all_subsystems > 1 ||
$ask_components && @all_subsystems == 1 && @all_components > 1 ||
$ask_components && @all_subsystems > 1)
{
ENV_say( 1, "Now select when/where this $ATT_TYPE_NAME must be used");
}




if ($ask_targets)
{
if (@gbs_targets == 0)
{
$targets_selector = 'None';
ENV_say( 1, "  No Targets");
} elsif (@gbs_targets == 1)
{
$targets_selector = 'ALL';
ENV_say( 1, "  Target @targets");
} else
{
ENV_say( 1, "  Possible Targets:",
"  @gbs_targets");
$targets_selector = ASK_value_from_menu( "  Select Targets-Selector",
0, qw( All Only Exclude));
my @selectors;
if ($targets_selector eq 'Only' || $targets_selector eq 'Exclude')
{
push @selectors, ASK_values_from_list( "$targets_selector Targets:", '', \@gbs_targets, -1);
}
@targets = select_selector_atts( $targets_selector, \@selectors, \@gbs_targets);
ENV_say( 1, "  Targets: @targets");
}
}




if ($ask_subsystems)
{
if (@all_subsystems == 0)
{
$subsystems_selector = 'None';
ENV_say( 1, "  No SubSystems");
} elsif (@all_subsystems == 1)
{
$subsystems_selector = 'All';
ENV_say( 1, "  SubSystem: @all_subsystems");
} else
{
ENV_say( 1, "  Possible SubSystems:",
"  @all_subsystems");
$subsystems_selector = ASK_value_from_menu( "  Select SubSystems-Selector",
0, qw( All Only Exclude None));
my @selectors;
if ($subsystems_selector eq 'Only' || $subsystems_selector eq 'Exclude')
{
push @selectors, ASK_values_from_list( "  $subsystems_selector SubSystems:", '', \@all_subsystems, -1);
}
@subsystems = select_selector_atts( $subsystems_selector, \@selectors, \@all_subsystems);
ENV_say( 1, "  SubSystems: @subsystems");
}
} else
{
if (@all_subsystems == 0)
{
$subsystems_selector = 'None';
} else
{
$subsystems_selector = 'All';
}
}




if ($ask_components)
{
if (@subsystems == 0)
{
ENV_say( 1, "  No Components");
$components_selector = 'None';
} elsif (@subsystems == 1)
{
@all_components = GBSGLO_components( $subsystems[0]);
@components = @all_components;
if (@all_components == 0)
{
$components_selector = 'None';
ENV_say( 1, "  No Components");
} elsif (@all_components == 1)
{
$components_selector = 'All';
ENV_say( 1, "  Component: @all_components");
} else
{
ENV_say( 1, "  Possible Components:",
"  @all_components");
$components_selector = ASK_value_from_menu( "  Select Components-Selector",
0, qw( All Only Exclude None));
my @selectors;
if ($components_selector eq 'Only' || $components_selector eq 'Exclude')
{
push @selectors, ASK_values_from_list( "  $components_selector Components:", '', \@all_components, -1);
}
@components = select_selector_atts( $components_selector, \@selectors, \@all_components);
ENV_say( 1, "  Components: @components");
}
} else
{
ENV_say( 1, "  Multiple SubSystems are involved: use wildcards for Components Selection");
$components_selector = ASK_value_from_menu( "  Select Components-Selector",
0, qw( All Only Exclude None));
if ($components_selector eq 'Only' || $components_selector eq 'Exclude')
{



@components = ASK_wild_words( "  $components_selector (wild) Components:", '', -1, -1);
}
}
}




ENV_say( 0, '');
ENV_say( 1, "Summary for $ATT_TYPE_NAME '$att_name':");
ENV_say( 0, "  Copy From  : $org_att")				if ($ask_new && $ACTION eq 'COPY');
ENV_say( 0, "  Plugin     : $plugin_name")				if ($ask_new);
ENV_say( 0, "  Targets    : $targets_selector: @targets")		if ($ask_targets);
ENV_say( 0, "  SubSystems : $subsystems_selector: @subsystems")	if ($ask_subsystems);
ENV_say( 0, "  Components : $components_selector: @components")	if ($ask_components);
} else
{
ENV_say( 0, '');
ENV_say( 1, 'Retry');
$all_ok = 0;
}
} while (!$all_ok || ASK_YNQ( 'OK?', 'Y', 1) eq 'N');

$targets_ref = \@targets
if ($ask_targets);
$subsys_ref = \@subsystems
if ($ask_subsystems);
if ($ask_components)
{
$comps_selector = $components_selector;
$components_sel_ref = [ @components ];
}
return ($plugin_name, $org_att, $targets_ref, $subsys_ref, $comps_selector, $components_sel_ref);
}





sub select_plugin_name()
{
my $plugin_name;

get_plugins();




my @menu_refs;
push @menu_refs, ([ '' ]);		# Menu entry 0 : <none> Nothing

foreach my $ref (@{$PLUGINS{$ATT_TYPE}})
{
my ($name, $description) = @{$ref};
my $display_text = sprintf( "%-*s: %s", $PLUGINS_MAX_NAME, $name, $description);
push @menu_refs, [ $display_text, $name ];
}




my $index = ASK_index_from_menu( "  Select $ATT_TYPE plugin",
\$DEFAULT_GEN_NAME, ( @menu_refs));
$plugin_name = $menu_refs[$index]->[1];	# $name
ENV_say( 1, "  Selected: $plugin_name")
if (defined $plugin_name);

return $plugin_name;
}





sub select_org_att()
{
my ($att_name, $plugin_name);

($att_name, $plugin_name) = select_att( copy => '');  # force ask

$att_name = undef
if ($att_name eq '');

return ($att_name, $plugin_name);
}




sub do_install($$$)
{
my ($plugin_name,
$org_att,
$att_name,
) = @_;

my $plugin_filespec = FSPEC_att_file( $ATT_FILE, $GBS_ROOT_PATH, $att_name);

print "ACTION=$ACTION plugin_filespec=$plugin_filespec\n";
if ($ACTION eq 'P_REF')
{
if (-e $plugin_filespec)
{
SCM_assert_co_file( $plugin_filespec);
ENV_unlink( $plugin_filespec);
}
FCREATE_att_ref_file( $ATT_FILE, $GBS_ROOT_PATH, $att_name, $plugin_name);
do_plugin_file( $plugin_name, $att_name, 'I');
} elsif ($ACTION eq 'P_ORG')
{
do_plugin_file( $plugin_name, $att_name, 'U');
} elsif ($ACTION eq 'O_NEW')
{
FCREATE_att_file( $ATT_FILE, $GBS_ROOT_PATH, $att_name, $plugin_name);
} elsif ($ACTION eq 'O_REF')
{
if (-e $plugin_filespec)
{
SCM_assert_co_file( $plugin_filespec);
ENV_unlink( $plugin_filespec);
}
FCREATE_att_ref_file( $ATT_FILE, $GBS_ROOT_PATH, $att_name, $plugin_name);
} elsif ($ACTION eq 'COPY')
{
my $from_plugin_att = $org_att;
my $from_path = "$GBS_ATTS_PATH/$from_plugin_att";
my @files = SLURP_dir_files( $from_path, 0);
if (@files)
{
ENV_say( 1, "$ATT_TYPE_NAME $from_plugin_att contains the following files",
"@files");
my @to_file_specs;
foreach my $file (@files)
{
my $from_spec = "$from_path/$file";
my $to_spec = "$GBS_ATT_PATH/$file";
ENV_say( 0, " copy $from_spec -> $GBS_ATT_PATH/");
SCM_assert_co_file( $to_spec);			# TBS add ATT-translate
FILE_copy_translate( $from_spec, $to_spec);
push @to_file_specs, $to_spec;
}
ENV_say( 1, scalar @to_file_specs . " copied");
SCM_add_txt_files( \@to_file_specs, 0, 1)
} else
{
ENV_say( 1, "No files in $ATT_TYPE_NAME $from_plugin_att");
}
} else
{
ENV_fatal( "Impossible $ACTION");
}




if (! -e $plugin_filespec)
{



FCREATE_att_file( $ATT_FILE, $GBS_ROOT_PATH, $att_name, $plugin_name);
}
SCM_add_txt_files( $plugin_filespec, 0, 1);		# $prevalidated, $verbose_level
}




sub get_plugins()
{
if (! $PLUGINS{$ATT_TYPE})
{
my @plugin_refs = PLG_get_plugins( $ATT_TYPE);





$PLUGINS_MAX_NAME = 0;
foreach my $ref (@plugin_refs)
{
my $plugin_name = $ref->[0];
my $name_length = length( $plugin_name);
$PLUGINS_MAX_NAME = $name_length
if ($name_length > $PLUGINS_MAX_NAME);
}
$PLUGINS{$ATT_TYPE} = [ @plugin_refs ];
}
}




sub do_plugin_file($$$)
{
my ($plugin_name,
$att_name,
$install_or_update, # 'I' or 'U'
) = @_;

my $to_dir = "$GBS_ATTS_PATH/$att_name";
my $plugins_root_path = "$GBS_SCRIPTS_PATH/plugins/$ATT_TYPE";
my $from_path = "$plugins_root_path/$plugin_name";

my @data_refs = PLG_get_plugin_data( $plugin_name, $ATT_TYPE, $install_or_update);




if (@data_refs)
{



ENV_say( 1, "Plugin Data overview:");
foreach my $ref (@data_refs)
{
my ( $func, $command, $ask_default, @args ) = @{$ref};
if ($command eq 'C')
{



ENV_say( 0, "  Copy @args");
} elsif ($command eq 'R')
{



ENV_say( 0, "  Run @args");
} elsif ($command eq 'D')
{



ENV_say( 0, "  Create Dir(s) @args");
} else # ($command eq 'E')
{



ENV_say( 0, "  Echo @args");
}
}




ENV_say( 1, "Executing...");
foreach my $ref (@data_refs)
{
my ( $func, $command, $ask_default, @args ) = @{$ref};
if ($command eq 'C')
{



my ($from_file, $to_file) = @args;	    # names may contain dirs!
my @file_refs;

if (ENV_is_wildcard( $from_file))
{
my @from_files = glob( "$from_path/$from_file");
foreach my $from_file (@from_files)
{
$from_file =~ s!^$from_path/!!;
my $file = ENV_split_spec_f( $from_file);
push @file_refs, [ $file, $from_file, $file ];
}
} else
{
my $file = ENV_split_spec_f( $from_file);
$to_file = $file
if (! $to_file);
push @file_refs, [ $file, $from_file, $to_file ];
}
my @from_file_names = map { $_->[1] } @file_refs;
ENV_say( 1, "Files to copy:");
ENV_say( 0, "    @from_file_names");

my $ans = ($ask_default eq '+') ? 'A' : $ask_default;
foreach my $ref (@file_refs)
{
my ($file, $from_file, $to_file) = @{$ref};
my $from_file_spec = "$from_path/$from_file";
my $to_file_spec   = "$to_dir/$to_file";

$ans = ASK_YNAE( "Copy file: $from_file?", $ask_default)
if ($ans ne 'A');
if ($ans eq 'Y' || $ans eq 'A')
{
copy_1_file( $file, $from_file_spec, $to_file_spec, $plugin_name);
} elsif ($ans eq 'E')
{
last;
}
}
} elsif ($command eq 'R')
{



my ($script, $script_spec) = @args;

my $ans = 'Y';
if ($ask_default ne '+')
{
$ans = ASK_YN( 'Run Installer?', $ask_default);
}
if ($ans eq 'Y')
{
ENV_say( 1, "Executing $script...");
my $command_ref = ENV_command_ref( $script_spec, [ $func, $from_path, $to_dir ]);
my $rc = ENV_system( $command_ref, undef, undef);
ENV_error( "Install failed")
if ($rc != 0);
}
} elsif ($command eq 'D')
{



my $dir = $args[0];
my $dir_spec = "$to_dir/$dir";
if (!-d $dir_spec)
{
ENV_say( 1, "Creating Directory '$dir'");
SCM_add_dirs( [ $dir_spec ], \&FILES_get_ignore_spec, 0, 1);	    # $pre_validated, $verbose
}
} else # ($command eq 'E')
{



ENV_say( 1, "$args[0]");
}
}
} else
{

}
}




sub copy_1_file($$$$)
{
my ($file_name,
$in_file_spec,
$to_file_spec,
$plugin_name,
) = @_;

my $ans = 'Y';
if (-e $to_file_spec)
{
$ans = ASK_YN( "File already exists. Replace?", "N");
}
if ($ans eq 'Y')
{
SCM_assert_co_file( $to_file_spec);

my $file_name_uc = uc $file_name;
$file_name_uc =~ s/\..*$//;		    # remove .type
my $type_name_uc = uc $ATT_TYPE_NAME;    # TARGET AUDIT TOOL
FILE_copy_template( $in_file_spec, $to_file_spec,
( SCRIPT => $file_name_uc, $type_name_uc => $plugin_name ));

ENV_chmod ('ug+x', $to_file_spec)
if (ENV_is_shell_file( $to_file_spec));
SCM_add_txt_files( $to_file_spec, 1, 1);	# $prevalidated, $verbose_level
ENV_say( 1, "File copied");
}
}




sub select_selector_atts($$$)
{
my ($selector,			# 'All', 'Only', "Exclude', 'None'
$selections_ref,		# Do not modify! ??
$all_atts_ref,
) = @_;
my @selection;

my @selection_list = @{$selections_ref};    # copy. We do not want to modify
if ($selector eq 'All')
{
@selection =  @{$all_atts_ref};
} elsif ($selector eq 'Only')
{
@selection = ENV_wildcards( \@selection_list, $all_atts_ref, undef);
} elsif ($selector eq 'Exclude')
{
@selection = ENV_not_wildcards( \@selection_list, $all_atts_ref);
} elsif ($selector eq 'None')
{
@selection = ();
} else
{
ENV_fatal( "Impossible selector: $selector");
}


return @selection;
}




sub add_dirs($)
{
my ($dirs_ref) = @_;

my @dirs = LIST_unique( $dirs_ref);		# remove duplicates
my @spec_refs = SCM_get_states( \@dirs);	# -2=no such file, -1=not in SCM, 0=checked-in, 1=checked-out

my @new_dirs;
foreach my $ref (@spec_refs)
{
my ($spec, $state) = @{$ref};
push @new_dirs, $spec
if ($state < 0);
}
ENV_debug( undef, "New:", @new_dirs);
SCM_add_dirs( \@new_dirs, \&FILES_get_ignore_spec, 0, 1);	    # $pre_validated, $verbose_level
}




sub system_add($$)
{
my ($new_att,
$plugin,
) = @_;




if (grep( $_ eq $new_att, @GBS_ATT_VALUES))
{
ENV_error( "SYSTEM Add $ATT_TYPE_ID $new_att: not executed. Already present");
} else
{
SYSTEM_add_os_ref( $GBS_ROOT_PATH, '.', $ATT_TYPE_ID => [ $new_att, $plugin ]);
@GBS_ATT_VALUES = sort( ( @GBS_ATT_VALUES, $new_att ));
system_store();
}
}




sub system_store()
{




SYSTEM_write();    # Includes backup and SCM_checkout

@GBS_ALL_ATT_VALUES = SYSTEM_get_os_ref_names( $GBS_ROOT_PATH, '*', $ATT_TYPE_ID);




GBSENV_setenv( $ATTS_ENV_NAME => [ @GBS_ATT_VALUES ]);
GBSENV_setenv( $ALL_ATTS_ENV_NAME => [ @GBS_ALL_ATT_VALUES ] );
}




sub setup_type($)
{
my ($att_type) = @_;

if (defined $att_type)
{
($ATT_TYPE, $ATT_TYPE_NAME, $ATT_FILE, $ATT_TYPE_ID) = @{$PLUGIN_SETUP{$att_type}};

$ATT_TYPE_NAME_S = "${ATT_TYPE_NAME}s";

$GBS_SCRIPTS_PLUGINS_PATH   = "$GBS_SCRIPTS_PATH/plugins/$ATT_TYPE";
$GBS_ATTS_PATH		    = "$GBS_ROOT_PATH/sys$ATT_TYPE";	# e.g.: sysaudit systarget systool

my $att_type_id_uc = uc $ATT_TYPE_ID;
$ATTS_ENV_NAME	    = "GBS_$att_type_id_uc";		# 'GBS_AUDITS',     'GBS_TARGETS',     'GBS_TOOLS'
$ALL_ATTS_ENV_NAME  = "GBS_ALL_$att_type_id_uc";	# 'GBS_ALL_AUDITS', 'GBS_ALL_TARGETS', 'GBS_ALL_TOOLS'
$ATT_ENV_NAME	    = substr( $ATTS_ENV_NAME, 0, -1);	# 'GBS_AUDIT',      'GBS_TARGET',      'GBS_TOOL'

$GBS_ATT_VALUE = ENV_getenv( $ATT_ENV_NAME);
@GBS_ATT_VALUES = ENV_getenv( $ATTS_ENV_NAME);
@GBS_ALL_ATT_VALUES = ENV_getenv( $ALL_ATTS_ENV_NAME);
} else
{
($ATT_TYPE, $ATT_TYPE_NAME, $ATT_FILE, $ATT_TYPE_ID) = ();

$ATT_TYPE_NAME_S = undef;

$GBS_SCRIPTS_PLUGINS_PATH = undef;
$GBS_ATTS_PATH = undef;

$ATTS_ENV_NAME	= undef;
$ALL_ATTS_ENV_NAME = undef;
$ATT_ENV_NAME = undef;

$GBS_ATT_VALUE = undef;
@GBS_ATT_VALUES = undef;
@GBS_ALL_ATT_VALUES = undef;
}

setup_plugin( undef);   # Clear $PLUGIN_NAME and $GBS_SCRIPTS_PLUGIN_PATH
setup_att( undef);	    # Clear $ATT_NAME and GBS_ATT_PATH
}




sub setup_plugin($)
{
my ($plugin_name) = @_;

if (defined $plugin_name)
{
$PLUGIN_NAME = $plugin_name;
$GBS_SCRIPTS_PLUGIN_PATH = "$GBS_SCRIPTS_PATH/plugins/$ATT_TYPE/$PLUGIN_NAME";
} else
{
$PLUGIN_NAME = undef;
$GBS_SCRIPTS_PLUGIN_PATH = undef;
}
}




sub setup_att($)
{
my ($att_name) = @_;

if (defined $att_name)
{
$ATT_NAME = $att_name;
$GBS_ATT_PATH = "$GBS_ATTS_PATH/$att_name";
} else
{
$ATT_NAME = undef;
$GBS_ATT_PATH = undef;
}
}




sub do_gbsext($)
{
my ($plugin_name,
) = @_;




my $plugin_uc = uc $plugin_name;
my $path_env_name = "GBSEXT_${plugin_uc}_PATH";
my $rel_env_name = "GBSEXT_${plugin_uc}_REL";
if (ENV_getenv( $path_env_name) eq '' || ENV_getenv( $rel_env_name) eq '')
{
ENV_warn( "You must 'gbsedit switch.gbs' to define $path_env_name and $rel_env_name");
if ( ASK_YN( "Do you want to startup gbsedit now (in a separate window)?", 'Y') eq 'Y')
{
my $switch_filespec = FSPEC_switch( 'switch.gbs', $GBS_ROOT_PATH);
GBSSCM_preset( 0);
GBSSCM_connect();
SCM_assert_co_file( $switch_filespec);
ENV_say( 1, "Editing '$switch_filespec'...");
RUN_editor( $switch_filespec, '');
ASK_pause( "When done...");
ENV_say( 1, "*** You need to execute an 'swr .' for this change to become effective ***");
}
}




my $gbs_silo_cache_path = "$GBS_ROOT_PATH/silo/.gbs/cache";
my $gbs_silo_plugin_path = "$gbs_silo_cache_path/$plugin_name";
if (! -e $gbs_silo_plugin_path)
{
ENV_say( 1, "Creating Silo Plugin dir...");
SCM_add_dirs( $gbs_silo_plugin_path, \&FILES_get_ignore_spec, 0, 1);	    # $pre_validated, $verbose_level
}
}

return 1;
