#===============================================================
#
#   pclintglo.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#   Generic package for qac and qac++
#===============================================================

package pclintglo;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
PCLINTGLO_init
PCLINTGLO_get_msg_filespecs
PCLINTGLO_read_msg_file
);

use strict;
use integer;

use glo::env;
use glo::slurp;
use scadef;




sub PCLINTGLO_init();
sub PCLINTGLO_get_msg_filespecs();
sub PCLINTGLO_read_msg_file($);

sub get_msg_level($);




my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');




my $GBSEXT_PLUGIN_PATH;

my $MSG_FILE_SPEC;		# "$GBSEXT_PLUGIN_PATH/bin/$PLUGIN_NAME.msg";		# must exist




sub PCLINTGLO_init()
{
$GBSEXT_PLUGIN_PATH = ENV_getenv( "GBSEXT_PCLINT_PATH");
ENV_fatal( "pclint: GBSEXT_PCLINT_PATH ($GBSEXT_PLUGIN_PATH): No such directory")
if (! -d $GBSEXT_PLUGIN_PATH);

$MSG_FILE_SPEC = "$GBSEXT_PLUGIN_PATH/msg.txt";		# must exist
}




sub PCLINTGLO_get_msg_filespecs()
{
return ($MSG_FILE_SPEC);
}




sub PCLINTGLO_read_msg_file($)
{
my ($help_info_wanted) = @_;    # Bool
my @msg_def_refs;






my @msg_level_refs = SCADEF_get_levels( $GBS_AUDIT);



my @level_texts;
map { $level_texts[$_->[0]] = $_->[2] } @msg_level_refs;	# $level $text




my $in_lines = SLURP_file( $MSG_FILE_SPEC) . '9999 dummy';
my @msg_lines = $in_lines =~ /\n\d{1,4}\*?\s+.*?(?=\n\d)/gs;
ENV_fatal( "Cannot parse paragraphs in $MSG_FILE_SPEC")
if (@msg_lines == 0);







foreach my $line (@msg_lines)
{

my ($msg_nr, $msg_text) = $line =~ /^\n(\d+)\*?\s+(.*?)\s+--\s+/s;
ENV_fatal( "Cannot parse line in $MSG_FILE_SPEC", "Line: $line")
if (!defined $msg_nr);
$msg_text =~ s/\s*\n\s+/ /sg;	    # splice lines

my $level = get_msg_level( $msg_nr);
my $level_text = $level_texts[$level];	# $text
my $help_file_url = "msg_help.html#$msg_nr";
if ($help_info_wanted)
{
my ($heading) = $line =~ /\d+\s*(.*?)  --/s;	# note: 2 spaces before '--'!
$heading = '***'
if (!defined $heading);

push @msg_def_refs, [ $msg_nr, $level, $level_text, $msg_text, $help_file_url, $heading, $line ];
} else
{
push @msg_def_refs, [ $msg_nr, $level, $level_text, $msg_text, $help_file_url ];
}
}

return @msg_def_refs;
}




sub get_msg_level($)
{
my ($msg_nr) = @_;
my $msg_level;

$msg_level = $msg_nr * 1;	    # make it integer
$msg_level -= 1000
if ($msg_level >= 1000);    # C++ adds 1000
if ($msg_level <= 199)
{
$msg_level = 1;		    # Syntax Error
} elsif ($msg_level <= 399)
{
$msg_level = 0;		    # Internal Error
} elsif ($msg_level <= 699)
{
$msg_level = 2;		    # Warning
} elsif ($msg_level <= 899)
{
$msg_level = 3;		    # Info
} else  #if ($msg_level <= 999)
{
$msg_level = 4;		    # Elective
}

return $msg_level;
}

return 1;


