#===============================================================
#
#   pclintmaint.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

print "*PCLINTMAINT*\n" if ($ENV{GBSDEBUG_FILE});
use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::args;
use glo::ask;
use gbsenv;
use run;
use scahelp;
use scamaint;
use lib "$ENV{GBS_SCRIPTS_PATH}/plugins/audit/pclint/bin";
use pclintglo;




sub do_show_help($$$);
sub do_convert_settings($$$);
sub convert_settings_line($$);

sub handle_msg_nr($$$$$);




my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_AUDIT_PLUGIN = ENV_getenv( 'GBS_AUDIT_PLUGIN');	    # pclint

my %ALL_MSG_NRS;

my @ALL_MSG_NRS;







$| = 1;
my $RC = 0;
GBSENV_init();

ARGS_get( 0, 'perl pclintmaint.pl', undef);

ENV_say( 1, "$GBS_AUDIT [ $GBS_AUDIT_PLUGIN ]");

{
SCAMAINT_init();

my @main_menu_items = (
[ 'Show Help file (html)', \&do_show_help ],
[ "Convert $GBS_AUDIT_PLUGIN warning settings file to GBSSCA settings file", \&do_convert_settings ],
);
ASK_menu( 'Select function to perform', \@main_menu_items, undef);
}




ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 1);
}




sub do_show_help($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref,
) = @_;

ENV_say( 1, 'Show Msg Help');

my $help_filespec = SCAHELP_get_spec( 'PCLINT', ENV_getenv( 'GBSEXT_PCLINT_REL'));
if (-e $help_filespec)
{
RUN_browser( $help_filespec);
} else
{
ENV_warn( 'Msg Help file does not exist');
}
}




sub do_convert_settings($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref,
) = @_;

ENV_say( 1, 'Convert settings file');

PCLINTGLO_init();

foreach my $msg_ref (PCLINTGLO_read_msg_file( 0))		# no help data wanted
{

my $msg_nr = $msg_ref->[0];
$ALL_MSG_NRS{$msg_nr} = $msg_ref->[3];	# $msg_text
}
@ALL_MSG_NRS = keys %ALL_MSG_NRS;

SCAMAINT_convert_settings( \&convert_settings_line, '//');

%ALL_MSG_NRS = ();
@ALL_MSG_NRS = ();
}




sub convert_settings_line($$)
{
my ($inline,
$line_nr,
) = @_;
my @outlines;


















if (substr( $inline, 0, 1) eq '-' || substr( $inline, 0, 1) eq '+')
{
my ($valid_part, $comments) = split( /\s*\/\//, $inline, 2);

my ($plus_or_minus, $msg_nr) = $valid_part =~ /^(-|\+)e([0-9?*]+)/;	    # -e365   // comments
if (defined $msg_nr)
{
my $sca_option;
if (ENV_is_wildcard( $msg_nr))
{
my @msg_nrs = ENV_wildcard( $msg_nr, \@ALL_MSG_NRS);
push @outlines, "# Wildcard: $inline";
push @outlines, map { handle_msg_nr( $plus_or_minus, $_, $comments, $line_nr, \$inline)} @msg_nrs;
} else
{
push @outlines, handle_msg_nr( $plus_or_minus, $msg_nr, $comments, $line_nr, \$inline);
}
} else
{
push @outlines, "# Ignored: $inline";
ENV_say( 1, "Ignored", sprintf( "%6d  %s", $line_nr, $inline));
}
} else
{
push @outlines, "# Ignored: $inline";
ENV_say( 1, "Line ignored (does not start with '-')", sprintf( "%6d  %s", $line_nr, $inline));
}

return @outlines;
}




sub handle_msg_nr($$$$$)
{
my ($plus_or_minus,
$msg_nr,
$comments,	# may be undef
$line_nr,
$inline_ref,
) = @_;
my @outlines;

my $msg_text = $ALL_MSG_NRS{$msg_nr};
if (defined $msg_text)
{
if (defined $comments)
{
push @outlines, "${plus_or_minus}E $msg_nr\t\t# $comments";
} else
{
push @outlines, "${plus_or_minus}E $msg_nr\t\t# $msg_text";
}
} else
{
push @outlines, "#${plus_or_minus}E $msg_nr";
ENV_say( 1, "Msg-nr $msg_nr not found in msg file", sprintf( "%6d  %s", $line_nr, $$inline_ref));
}

return @outlines;
}


