#===============================================================
#
#   prqamaint.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

print "*PRQAMAINT* @ARGV\n" if ($ENV{GBSDEBUG_FILE});
use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::args;
use glo::ask;
use run;
use gbsenv;
use scamaint;
use lib "$ENV{GBS_SCRIPTS_PATH}/plugins/audit/qac/bin";
use prqaglo;




sub do_show_help($$$);
sub do_convert_settings($$$);
sub convert_settings_line($$);
sub parse_comma_range($$$$);
sub parse_threshold($$$);




my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_AUDIT_PLUGIN = ENV_getenv( 'GBS_AUDIT_PLUGIN');	    # qac or qacpp

my %ALL_MSG_NRS;








$| = 1;
my $RC = 0;
GBSENV_init();

my ($ARG_PLUGIN_NAME) = ARGS_get( 1, 'perl prqamaint.pl', undef);

ENV_say( 1, "$GBS_AUDIT [ $GBS_AUDIT_PLUGIN ]");

{
SCAMAINT_init();

my @main_menu_items = (
[ 'Show Help file (html)', \&do_show_help ],
[ "Convert $GBS_AUDIT_PLUGIN warning settings file to GBSSCA settings file", \&do_convert_settings ],
);
ASK_menu( 'Select function to perform', \@main_menu_items, undef);
}




ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 1);
}




sub do_show_help($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref,
) = @_;

ENV_say( 1, 'Show Help');

my $gbsext_plugin_path = ENV_getenv_perl_path( "GBSEXT_${ARG_PLUGIN_NAME}_PATH");

RUN_browser( "$gbsext_plugin_path/help/messages/messages.htm");
}




sub do_convert_settings($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref,
) = @_;

ENV_say( 1, 'Convert settings file');

PRQAGLO_init( $GBS_AUDIT_PLUGIN);

foreach my $msg_ref (PRQAGLO_read_msg_file())
{

my $msg_nr = $msg_ref->[0];
$ALL_MSG_NRS{$msg_nr} = $msg_ref->[3];	# $msg_text
}

SCAMAINT_convert_settings( \&convert_settings_line, '*');

%ALL_MSG_NRS = ();
}




sub convert_settings_line($$)
{
my ($inline,
$line_nr,
) = @_;
my @outlines;




















if (substr( $inline, 0, 1) eq '-')
{
if ($inline =~ /^-echo\s/i)
{
push @outlines, "# Ignored: $inline";
ENV_say( 1, "-echo not supported", sprintf( "%6d  %s", $line_nr, $inline));
} else
{

my @opts = split( /\s+-/, " $inline");
shift @opts;	    # skip the added first ' '
foreach my $opt (@opts)
{
my ($key, $value) = split( /\s+/, $opt, 2);
my $key_lc = lc $key;
if ($key_lc eq 'o' || $key_lc eq 'only')
{

push @outlines, parse_comma_range( '+', $value, $line_nr, \$inline);
} elsif ($key_lc eq 'n' || $key_lc eq 'no' || $key_lc eq 'nomsg')
{

push @outlines, parse_comma_range( '-', $value, $line_nr, \$inline);
} elsif ($key_lc eq 'q' || $key_lc eq 'quiet')
{

push @outlines, "+Q $value";
} elsif ($key_lc eq 'forget' || $key_lc eq 'forgetall')
{

my $value_lc = lc $value;
if ($value_lc eq 'o' || $value_lc eq 'only')
{
push @outlines, '-E ALL';
} elsif ($value_lc eq 'no' || $value_lc eq 'nomsg')
{
push @outlines, '+E ALL';
} else
{
push @outlines, "# Ignored: $inline";
ENV_say( 1, "'-$key $value' not supported", sprintf( "%6d  %s", $line_nr, $inline));
}
} elsif ($key_lc eq 'thresh' || $key_lc eq 'thereshold')
{

push @outlines, parse_threshold( $value, $line_nr, \$inline);
} else
{
push @outlines, "# Ignored: $inline";
ENV_say( 1, "-$key not supported", sprintf( "%6d  %s", $line_nr, $inline));
}
}
}
} else
{
ENV_error( "Invalid line", sprintf( "%6d  %s", $line_nr, $inline));
}

return @outlines;
}




sub parse_comma_range($$$$)
{
my ($plus_or_minus,
$value,
$line_nr,
$inline_ref,
) = @_;
my @outlines;

foreach my $possible_range (split( ',', $value))
{
my @msg_nrs;
my ($msg_nr1, $msg_nr2) = $possible_range =~ /^(\d+)(-\d+|)$/;
if (defined $msg_nr1)
{
if ($msg_nr2 ne '')
{
$msg_nr2 = substr( $msg_nr2, 1);	# remove leading '-'
if ($msg_nr2 >= $msg_nr1)
{

foreach my $msg_nr ($msg_nr1..$msg_nr2)
{
push @msg_nrs, $msg_nr
if (exists( $ALL_MSG_NRS{$msg_nr}));
}
} else
{
ENV_error( "Range error $msg_nr2 < $msg_nr1", sprintf( "%6d  %s", $line_nr, $$inline_ref));
}
} else
{
@msg_nrs = ($msg_nr1);
}
} else
{
ENV_error( "'$value' Unknown syntax", sprintf( "%6d  %s", $line_nr, $$inline_ref));
}

foreach my $msg_nr (@msg_nrs)
{
my $msg_text = $ALL_MSG_NRS{$msg_nr};
if (defined $msg_text)
{
push @outlines, "${plus_or_minus}E $msg_nr\t\t# $msg_text";
} else
{
push @outlines, "#${plus_or_minus}E $msg_nr";
ENV_say( 1, "Msg-nr $msg_nr not found in msg file", sprintf( "%6d  %s", $line_nr, $$inline_ref));
}
}
}

return @outlines;
}




sub parse_threshold($$$)
{
my ($value,
$line_nr,
$inline_ref,
) = @_;
my @outlines;


my ($metric, $op, $met_value, $msgnr) = $value =~ /^ST(...)(<|<=|=|>|>=)([\d.]+)(:\d+|)$/;

if (defined $metric)
{

my $min = '';
my $max = '';
if ($op eq '<')
{
$max = $met_value;
$max--
unless ($max =~ /\./);
} elsif ($op eq '<=')
{
$max = $met_value;
} elsif ($op eq '=')
{
$min = $met_value;
$max = $met_value;
} elsif ($op eq '>')
{
$min = $met_value;
$min++
unless ($min =~ /\./);
} else #($op eq '>=')
{
$min = $met_value;
}
$max = ",$max"
if ($max ne '');
push @outlines, "+T $metric=$min$max$msgnr";
} else
{
push @outlines, "# Ignored: $$inline_ref";
ENV_error( "'Invalid syntax", sprintf( "%6d  %s", $line_nr, $$inline_ref));
}

return @outlines;
}


