#===============================================================
#
#   scahelp.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package scahelp;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
SCAHELP_get_spec
SCAHELP_init
SCAHELP_add_msg_text
SCAHELP_add_general_text
SCAHELP_write_help_file
);

use strict;
use integer;


use glo::spit;
use glo::file;
use glo::html;
use gbshtml;
use gbsenv;




sub SCAHELP_get_spec($$);
sub SCAHELP_init($$$);
sub SCAHELP_add_msg_text($$$$);
sub SCAHELP_add_general_text($$);
sub SCAHELP_write_help_file();




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();

my $GBS_SILO_CACHE_PATH = "$GBS_ROOT_PATH/silo/.gbs/cache";




my $PLUGIN_NAME;
my $PLUGIN_REL;
my $PLUGIN_NICE_NAME;

my $GBS_SILO_PLUGIN_PATH;   # $GBS_SILO_CACHE_PATH/$PLUGIN_NAME/$PLUGIN_REL

my $HELP_FILESPEC;	    # $GBS_SILO_PLUGIN_PATH/msg_help.html

my @HTML_LINES;
my $TITLE;




sub SCAHELP_get_spec($$)
{
my ($plugin_name,
$plugin_rel,
) = @_;


return "$GBS_SILO_CACHE_PATH/$plugin_name/$plugin_rel/msg_help.html";
}




sub SCAHELP_init($$$)
{
($PLUGIN_NAME,
$PLUGIN_REL,
$PLUGIN_NICE_NAME,
) = @_;

$GBS_SILO_PLUGIN_PATH = "$GBS_SILO_CACHE_PATH/$PLUGIN_NAME/$PLUGIN_REL";
$HELP_FILESPEC = "$GBS_SILO_PLUGIN_PATH/msg_help.html";

$TITLE = "$PLUGIN_NICE_NAME ($PLUGIN_REL) Warnings Helpfile";
push @HTML_LINES, GBSHTML_doc_start( $HELP_FILESPEC, "$TITLE - Created by GBS", 0);
push @HTML_LINES, GBSHTML_doc_top( $TITLE, ());
push @HTML_LINES, HTML_br();
}




sub SCAHELP_add_msg_text($$$$)
{
my ($msg_nr,
$level_text,
$heading,
$lines_ref,
) = @_;

push @HTML_LINES, HTML_h( 2, { id => $msg_nr }, "$msg_nr ($level_text) -- $heading");
push @HTML_LINES, HTML_pre_s();
my $line = join( "\n", @{$lines_ref});
push @HTML_LINES, HTML_pre_txt( $line);
push @HTML_LINES, HTML_pre_e();

}



sub SCAHELP_add_general_text($$)
{
my ($heading,
$lines_ref,
) = @_;

push @HTML_LINES, HTML_h( 2, $heading);
push @HTML_LINES, HTML_pre_s();
my $line = join( "\n", @{$lines_ref});
push @HTML_LINES, HTML_pre_txt( $line);
push @HTML_LINES, HTML_pre_e();
}





sub SCAHELP_write_help_file()
{
push @HTML_LINES, GBSHTML_doc_bottom( $TITLE, undef);
push @HTML_LINES, GBSHTML_doc_end();


my $fh = FILE_lock_ex_wait( "$HELP_FILESPEC.lck");
SPIT_file_nl( $HELP_FILESPEC, \@HTML_LINES);
FILE_unlock( $fh);

@HTML_LINES = ();
$TITLE = undef;

return $HELP_FILESPEC;
}

return 1;

