#===============================================================
#
#   scamaint.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package scamaint;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
SCAMAINT_init
SCAMAINT_convert_settings
);

use strict;
use integer;

use glo::env;
use glo::ask;
use glo::slurp;
use glo::spit;
use glo::scm;
use gbsenv;
use gbsglo;
use gbsscm;




sub SCAMAINT_init();
sub SCAMAINT_convert_settings($$);

sub write_file( $$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_AUDIT_PLUGIN = ENV_getenv( 'GBS_AUDIT_PLUGIN');	    # qac, qacpp, pclint

my $GBS_SYSAUDIT_PATH = "$GBS_ROOT_PATH/sysaudit";











sub SCAMAINT_init()
{
GBSSCM_preset( 0);
}




sub SCAMAINT_convert_settings($$)
{
my ($line_func_ref,	    # (@out_lines) = $line_func_ref->( $line, $line_nr)
$comment_chars,	    # E.g.: '*' or '//'
) = @_;




ENV_say( 1, "Please enter file-spec of the $GBS_AUDIT_PLUGIN settings file");
my $default_infile = '';
if (-e "D:/Work/Test/GBSEXT")	    # DEBUG
{
if ($GBS_AUDIT_PLUGIN eq 'qac')
{
$default_infile = "D:/Work/Test/GBSEXT/qac_pdsl/8.1-R/TOTAL_NOMSG.p_s";
} elsif ($GBS_AUDIT_PLUGIN eq 'qacpp')
{
$default_infile = "D:/Work/Test/GBSEXT/qacpp_pdsl/3.0-R/TOTAL_NOMSG.p_s";
} elsif ($GBS_AUDIT_PLUGIN eq 'pclint')
{
$default_infile = "D:/Work/Test/GBSEXT/pclint/8.00x/TOTAL_NOMSG.lnt";
}
}
my $infile_spec = ASK_file_path( "In File spec:", $default_infile, 1, 0);
if ($infile_spec ne '')
{



ENV_say( 1, "The output fille be written to \$GBS_SYSAUDIT_PATH/<audit>/sca_ALL.gbs",
"Additionally sca_<target>.gbs files will be created for every valid",
"Audit/Target combination",
"These files will contain a .include to the sca_ALL.gbs file");
my $default_outfile = "$GBS_SYSAUDIT_PATH/$GBS_AUDIT/sca_ALL.gbs";
my $outfile_spec = ASK_file_path( "Out File spec:", $default_outfile, 0, 0);
if ($outfile_spec ne '')
{











my $saved_nr_errors = ENV_get_nr_errors();
my $line_nr = 0;
my @outlines;
my $comment_chars_l = length( $comment_chars);
foreach my $inline (SLURP_file( $infile_spec))
{
$line_nr++;
if ($inline eq '')
{
push @outlines, $inline;
} elsif (substr( $inline, 0, $comment_chars_l) eq $comment_chars)
{
push @outlines, '#' . substr( $inline, $comment_chars_l);
} else
{
push @outlines,  $line_func_ref->( $inline, $line_nr);
}
}

map { print "$_\n" } @outlines;	# DEBUG
my $write_text = (-e $outfile_spec) ? 'Overwrite the existing' : 'Write the';
my $default_yn = 'Y';
if (ENV_get_nr_errors() > $saved_nr_errors)
{
ENV_say( 1, 'Errors found');
$default_yn = 'N';
}
if (ASK_YN( "$write_text $GBS_AUDIT sca_ALL.gbs?", $default_yn) eq 'Y')
{
write_file( $outfile_spec, \@outlines);
}
}
}




foreach my $target (ENV_getenv( 'GBS_ALL_TARGETS'))
{
if (GBSGLO_subsystems_for_audit( $GBS_AUDIT, $target))
{
my $sca_target_filespec = "$GBS_SYSAUDIT_PATH/$GBS_AUDIT/sca_$target.gbs";
my $default = (-e $sca_target_filespec) ? 'N' : 'Y';
if (ASK_YN( "Create $GBS_AUDIT sca_$target.gbs with reference to sca_ALL.gbs?", $default) eq 'Y')
{
my @lines = (
'',
'.include sca_ALL.gbs',
);
write_file( $sca_target_filespec, \@lines);
}
}
}
}




sub write_file($$)
{
my ($file_spec,
$lines_ref,
) = @_;

ENV_backup_file( $file_spec, '.sav', 1)	# exit on error
if (-e $file_spec);
SCM_assert_co_file( $file_spec);

my $file = ENV_split_spec_f( $file_spec);
my ($target) = $file =~ /sca_(.*)\./;
my @head = (
'#========================================================',
"#   [SYSAUDIT:${GBS_AUDIT}[$target] sca_$target.gbs",
'#========================================================',
);
my @tail = (
'',
'###EOF###',
);

SPIT_file_nl( $file_spec, \@head, $lines_ref, \@tail);
}

return 1;


