#===============================================================
#
#   scamsg.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package scamsg;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
SCAMSG_init
SCAMSG_new_needed
SCAMSG_add_msg
SCAMSG_write_file
SCAMSG_read_file
SCAMSG_get_msgs_and_levels
SCAMSG_msg_exists
SCAMSG_get_msg_data
SCAMSG_get_msg_data_element
);

use strict;
use integer;

use glo::env;
use glo::spit;
use glo::slurp;
use glo::file;
use gbsenv;




sub SCAMSG_init($$);
sub SCAMSG_new_needed($);
sub SCAMSG_add_msg($);
sub SCAMSG_write_file($);
sub SCAMSG_read_file();
sub SCAMSG_get_msgs_and_levels();
sub SCAMSG_msg_exists($);
sub SCAMSG_get_msg_data($);
sub SCAMSG_get_msg_data_element($$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SILO_CACHE_PATH = "$GBS_ROOT_PATH/silo/.gbs/cache";




my $GBS_SILO_PLUGIN_PATH;   # $GBS_SILO_CACHE_PATH/$plugin_name/$plugin_rel

my $MSG_FILESPEC;	    # $GBS_SILO_PLUGIN_PATH/msg.txt
my @MSG_REFS;

my %MSG_REFS;

my $MSG_FILE_CHANGED;




sub SCAMSG_init($$)
{
my ($plugin_name,
$plugin_rel,
) = @_;

$GBS_SILO_PLUGIN_PATH = "$GBS_SILO_CACHE_PATH/$plugin_name/$plugin_rel";
if (! -e $GBS_SILO_PLUGIN_PATH)
{
ENV_say( 1, "Creating Silo Plugin dirs...");
mkdir "$GBS_SILO_CACHE_PATH/$plugin_name";	# should already be there
mkdir "$GBS_SILO_CACHE_PATH/$plugin_name/$plugin_rel";
}

$MSG_FILESPEC  = "$GBS_SILO_PLUGIN_PATH/msg.txt";
}





sub SCAMSG_new_needed($)
{
my ($plugin_msg_files_ref,		# may also be scalar
) = @_;
my $new_needed = 0;




$plugin_msg_files_ref  = [ $plugin_msg_files_ref ]
if (!ref( $plugin_msg_files_ref));
foreach my $plugin_msg_filespec (@{$plugin_msg_files_ref})
{
if (ENV_file_is_newer( $plugin_msg_filespec, $MSG_FILESPEC))
{
$new_needed = 1;
last;
}
}

return $new_needed;
}






sub SCAMSG_add_msg($)
{
my ($msg_ref,
) = @_;


my $msg_nr = $msg_ref->[0];
push @MSG_REFS, $msg_ref;
$MSG_REFS{$msg_nr} = $msg_ref;
$MSG_FILE_CHANGED = 1;
}





sub SCAMSG_write_file($)
{
my ($force_write,
) = @_;


if ($MSG_FILE_CHANGED || $force_write)
{
ENV_say( 1, "Writing SCA Msg file...");

my @lines;
foreach my $msg_nr (sort keys %MSG_REFS)
{
push @lines, join( "\t", @{$MSG_REFS{$msg_nr}});
}
my $fh = FILE_lock_ex_wait( "$MSG_FILESPEC.lck");
SPIT_file_nl( $MSG_FILESPEC, \@lines);
FILE_unlock( $fh);
}
$MSG_FILE_CHANGED = 0;

return $MSG_FILESPEC;
}





sub SCAMSG_read_file()
{
my $fh = FILE_lock_sh_wait( "$MSG_FILESPEC.lck");
@MSG_REFS = map { [ split( "\t", $_) ] } SLURP_file( $MSG_FILESPEC);
FILE_unlock( $fh);
%MSG_REFS = map { $_->[0] => $_ } @MSG_REFS;
$MSG_FILE_CHANGED = 0;
}




sub SCAMSG_get_msgs_and_levels()
{
my @msg_level_refs;	    # [ $msg_nr, $level ]

foreach my $ref (@MSG_REFS)
{
push @msg_level_refs, [ $ref->[0], $ref->[1]];	# $msg_nr, $level
}

return @msg_level_refs;
}




sub SCAMSG_msg_exists($)
{
my ($msg_nr) = @_;


return (exists $MSG_REFS{$msg_nr}) ? 1 : 0;
}




sub SCAMSG_get_msg_data($)
{
my ($msg_nr) = @_;


return (wantarray) ? @{$MSG_REFS{$msg_nr}} : $MSG_REFS{$msg_nr};
}




sub SCAMSG_get_msg_data_element($$)
{
my ($msg_nr,
$index,
) = @_;


return $MSG_REFS{$msg_nr}->[$index];
}

return 1;


