#==============================================================
#
#   swa.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package swa;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
SWA_validate_dirs
SWA_set
);

use strict;
use integer;

use glo::env;
use gbsenv;
use gbsglo;
use gbsrc;
use system;




sub SWA_validate_dirs($$$$$);
sub SWA_set($$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();




sub SWA_set($$)
{
my ($audit,
$must_write,
) = @_;
my $audit_plugin;

ENV_debug( "Set $GBS_ROOT_PATH ($audit)");




$audit_plugin = ($audit eq '') ? '' : SYSTEM_get_os_ref_value( $GBS_ROOT_PATH, '.', 'audits', $audit);
ENV_setenv( GBS_AUDIT => $audit);
ENV_setenv( GBS_AUDIT_PLUGIN => $audit_plugin);

if (GBSENV_mode_is_interactive())
{



GBSRC_write_root( audit => $audit)
if ($must_write && GBSENV_mode_is_interactive());
}

return $audit_plugin;
}




sub SWA_validate_dirs($$$$$)
{
my ($root_path,
$subsys,
$component,
$audit,
$target,
) = @_;
my $new_audit = $audit;


if ($subsys ne '')
{



my $audit_dir = "$root_path/dev/$subsys/audit";
if (-d "$audit_dir/$audit")
{
if (-d "$audit_dir/$audit/$target")
{



if ( $component ne '' && GBSGLO_subsystem_is_full_gbs( $subsys, $root_path))
{
my $bld_dir = "$root_path/dev/$subsys/comp/$component/bld";
if (-d "$bld_dir/$audit")
{
if (!-d "$bld_dir/$audit/$target")
{
ENV_warn( "Component '$component' does not audit for Audit '$audit' and Target ($target)");
$new_audit = '';
}
} else
{
ENV_warn( "Component '$component' does not audit for Audit '$audit'");
$new_audit = '';
}
}
} else
{
ENV_warn( "SubSystem '$subsys' does not audit for Audit '$audit' and Target ($target)");
$new_audit = '';
}
} else
{
ENV_warn( "SubSystem '$subsys' does not audit for Audit '$audit'");
$new_audit = '';
}
}

return $new_audit;
}

return 1;



