#==============================================================
#
#   sws.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package sws;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
SWS_set
);

use strict;
use integer;

use glo::env;
use gbsenv;
use gbsrc;
use ssprop;




sub SWS_set($$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();




sub SWS_set($$)
{
my ($subsys,
$must_write,
) = @_;

ENV_debug( "Set $GBS_ROOT_PATH, $subsys");




my $ss_type;
my $subsys_path;
my @env_refs = (



[ SUBSYS	=> 1, 1, 'f', sub { return $subsys } ],
[ SSTYPE	=> 1, 1, 'f', sub { return $ss_type } ],
[ SUBSYS_PATH   => 1, 1, 'f', sub { return $subsys_path } ],
[ EXPORT_PATH   => 1, 1, 'p', undef ],
[ IMPORT_PATH   => 1, 1, 'p', undef ],
[ BUILD_PATH	=> 1, 1, 'p', undef ],
[ AUDIT_PATH    => 1, 1, 'p', undef ],
[ TOOLS_PATH    => 1, 1, 'p', undef ],


[ COMP_PATH	=> 1, 0, 'p', undef ],


[ APP_PATH	=> 0, 1, 'p', undef ],
);

if ($subsys eq '')
{



map { GBSENV_setenv( "GBS_$_->[0]", '') } @env_refs;
} else
{



$subsys_path = "$GBS_ROOT_PATH/dev/$subsys";
$ss_type = SSPROP_get_type( undef, $subsys);
my $is_full_gbs = ($ss_type eq 'GBS') ? 1 : 0;
foreach my $ref (@env_refs)
{
my ($name, $for_full, $for_not_full, $type, $arg) = @{$ref};

my $env_name = "GBS_$name";
my $value;
if ($is_full_gbs && $for_full ||
!$is_full_gbs && $for_not_full)
{
if ($type eq 'p')
{
my $path_part = lc( substr( $name, 0, -5));	    # minus "_PATH"
$value = "$subsys_path/$path_part";
} elsif ($type eq 'c')
{
$value = '';
} elsif ($type eq 'f')
{
$value = $arg->();
} else
{
ENV_fatal( "Impossible type ($type)");
}
} else
{
$value = '';
}

GBSENV_setenv( $env_name => $value );
}
}




GBSRC_write_root( subsys => $subsys)
if ($must_write && GBSENV_mode_is_interactive());
}

return 1;


