#===============================================================
#
#   tool.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package tool;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
TOOL_set_envs
TOOL_get_derived_items
TOOL_get_section_items
TOOL_get_plugin
);

use strict;
use integer;

use glo::env;
use gbsenv;
use gbssfile;




sub TOOL_set_envs($);
sub TOOL_get_derived_items($@);
sub TOOL_get_section_items($$@);
sub TOOL_get_plugin($);

sub read_tool($);





my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SYSTOOLS_PATH = "$GBS_ROOT_PATH/systools";

my $CUR_TOOL = '';
my $CUR_TOOL_REF;

my %TOOLS = ();




























my %SETUP_DEFS = (
NICE_NAME	    => [ 'ss' , undef,     0, undef, 1,			 undef, undef ],
);

my %INIT_DEFS = (
SET_W	    => [ 'aa' , [],	   0, 1,     1,			 undef,	undef ],
SET_X	    => [ 'aa' , [],	   0, 1,     1,			 undef,	undef ],
SET		    => [ 'aa' , [],	   0, 2,     1,			 undef,	undef ],
SETPATH	    => [ 'aa' , [],	   0, 3,   [1],			 undef,	undef ],
CALL	    => [ 'aC' , [],	   0, 4, undef,			 undef,	undef ],    # Not implemented
);










my %SECTION_DEFS = (
SETUP	=> [ 'N', 0, 1, 0,   \%SETUP_DEFS,  undef,	  undef ],
INIT	=> [ 'N', 0, 0, 0,   \%INIT_DEFS,  undef,	  undef ],
);




sub TOOL_set_envs($)
{
my ($tool) = @_;

read_tool( $tool);




if (grep( $_ eq $tool, ENV_getenv( 'GBS_TOOLS')))
{
GBSSFILE_set_init_envs( $CUR_TOOL_REF);
} else
{
ENV_say( 1, "Tool: '$tool' does not run for this platform\n");
}
}




sub TOOL_get_derived_items($@)
{
my ($tool,
@items,
) = @_;
my @values = ();


read_tool( $tool);

return GBSSFILE_get_section_items( $CUR_TOOL_REF, '.', @items);
}




sub TOOL_get_section_items($$@)
{
my ($tool,
$section,
@items,
) = @_;
my @values = ();


read_tool( $tool);

return GBSSFILE_get_section_items( $CUR_TOOL_REF, $section, @items);
}




sub TOOL_get_plugin($)
{
my ($tool) = @_;


read_tool( $tool);

return $CUR_TOOL_REF->{'.'}->{PLUGIN_DIR};
}




sub read_tool($)
{
my ($tool) = @_;	    # undef = use current, '' = use EnvVar GBS_TOOL

if (defined $tool)
{
$tool = ENV_getenv( 'GBS_TOOL')
if ($tool eq '');

if ($tool ne $CUR_TOOL)
{

$CUR_TOOL = $tool;
$CUR_TOOL_REF = $TOOLS{$CUR_TOOL};
}
if (!defined $CUR_TOOL_REF)
{

$TOOLS{$CUR_TOOL} = {};
$CUR_TOOL_REF = $TOOLS{$CUR_TOOL};




my $tool_file = "$GBS_SYSTOOLS_PATH/$CUR_TOOL/tool";
my @inc_path = ( "$GBS_SYSTOOLS_PATH/$CUR_TOOL",
$GBS_SYSTOOLS_PATH);
GBSSFILE_parse( $CUR_TOOL_REF, $tool_file, $tool, \@inc_path, \%SECTION_DEFS);

}
}
}

return 1;

