#!/bin/bash
#
#   Install.sh
#       Must NOT be sourced
#
if [[ ${BASH_SOURCE[0]} != "" ]]
then
    # on bash this will work even if the script is sourced
    #\echo "BASH_SOURCE[0]=${BASH_SOURCE[0]}"
    \pushd `dirname ${BASH_SOURCE[0]}` > /dev/null
else
    #\echo "No BASH_SOURCE"
    \pushd `dirname $0` > /dev/null
fi

#
#   Sanity Check: Assert TMPDIR
#
if [[ ! -z TMPDIR ]]
then
    \export "TMPDIR=/tmp/$USER"
    \mkdir "$TMPDIR" 2>/dev/null
fi

#
#   Setup the GUI environment (if available)
#
if [ "$XDG_DESKTOP_DIR" == '' ]
then
    #\echo "INSTALL_: Getting GUI info..."
    \test -f ${XDG_CONFIG_HOME:-~/.config}/user-dirs.dirs && source ${XDG_CONFIG_HOME:-~/.config}/user-dirs.dirs
    if [ "$XDG_DESKTOP_DIR" != '' ]
    then
        \echo "INSTALL_: GUI present ($XDG_DESKTOP_DIR)"
        \export XDG_DESKTOP_DIR
    fi
fi

#
#   Setup Perl
#
if [[ $GBS_PERL_PATH == '' ]]
then
    \export GBS_PERL_CMD=perl
else
    \export GBS_PERL_CMD=$GBS_PERL_PATH/bin/perl
    \echo "INSTALL_: Perl set to '$GBS_PERL_PATH/bin/perl'"
fi

#
#       Define GBS_SCRIPTS_PATH
#
for i in *; do
    if [ -d "$i" ]; then
        SUBDIR="$i"
    fi
done
if [[ "$SUBDIR" != "" ]]
then
    \echo "INSTALL_: GBS version: $SUBDIR"
    \export GBS_SCRIPTS_PATH="$PWD/$SUBDIR"
else
    #echo "INSTALL_: PWD=$PWD"
    echo "INSTALL_: *** In Development Environment ***"
    \pushd .. > /dev/null
    \export GBS_SCRIPTS_PATH="$PWD"
    \popd > /dev/null
fi
echo "INSTALL_: GBS_SCRIPTS_PATH set to $GBS_SCRIPTS_PATH"

#
#   Run Install
#       -q
#       --help
#
$GBS_PERL_CMD -w Install.pl $*

#
#   Exit
#
popd > /dev/null
read -p "INSTALL_: Hit <Enter> to finish (and maybe close window): "

###EOF###

